#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for dungeon generation
"""

from pyherc.data import Dungeon
from pyherc.aspects import logged

class DungeonGenerator():
    """
    This class is used to generate dungeon
    """
    @logged
    def __init__(self, creature_generator, item_generator,
                 level_generator):
        """
        Default constructor

        :param creature_generator: generator for creatures
        :param item_generator: generator for items
        :param level_generator: level generator for the first level
        """
        self.creature_generator = creature_generator
        self.item_generator = item_generator
        self.level_generator = level_generator

    @logged
    def generate_dungeon(self, model):
        """
        Generates start of the dungeon
        """
        model.dungeon = Dungeon()

        level = self.level_generator.generate_level(None)

        model.dungeon.levels = level

        portals = [x for x in level.portals
                   if x.exits_dungeon]
        exit = portals[0]

        level.add_creature(model.player, exit.location)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
AI routines for skeletons
"""

from pyherc.aspects import logged
from pyherc.ai.pathfinding import a_star
from pyherc.events import NoticeEvent, LoseFocusEvent

class SkeletonWarriorAI():
    """
    AI for skeleton warrior

    ..versionadded:: 0.7
    """
    @logged
    def __init__(self, character):
        """
        Default constructor

        :param character: character to connect
        :type character: Character
        """
        self.character = character

        self.mode = 'patrol'
        self.destination = None

    @logged
    def act(self, model, action_factory, rng):
        """
        Trigger this AI to assess the situation and act accordingly

        :param model: model where the character is located
        :type model: Model
        :param action_factory: factory to create actions
        :type action_factory: ActionFactory
        :param rng: random number generator
        :type rng: Random
        """
        del self.character.short_term_memory[:]

        if self.character.inventory.weapon == None:
            self._wield_weapon(action_factory)

        c_location = self.character.location
        p_location = model.player.location

        distance = ((c_location[0] - p_location[0]) ** 2 +
                    (c_location[1] - p_location[1]) ** 2) ** 0.5

        if distance < 4:
            if self.mode != 'combat':
                self.character.raise_event(
                                    NoticeEvent(character = self.character,
                                                target = model.player))
            self.mode = 'combat'
        else:
            if self.mode != 'patrol':
                self.character.raise_event(
                                LoseFocusEvent(character = self.character))
            self.mode = 'patrol'

        if self.mode == 'patrol':
            self._patrol(model, action_factory, rng)
        else:
            self._combat(model, action_factory, rng)

    @logged
    def _wield_weapon(self, action_factory):
        """
        Check if it is possible to wield a weapon and do so
        """
        weapons = [item for item in
                   self.character.inventory
                   if item.weapon_data != None]

        self.character.equip(weapons[0],
                             action_factory)

    @logged
    def _patrol(self, model, action_factory, rng):
        """
        Patrol around the level
        """
        character = self.character
        level = self.character.level

        while (self.destination == None
               or character.location == self.destination):

            self.destination = level.find_free_space()

        path, connections, updated = a_star(character.location,
                                            self.destination,
                                            level)

        next_tile = path[1]

        direction = self.find_direction(character.location,
                                        next_tile)

        if character.is_move_legal(direction,
                                   'walk',
                                   action_factory):
            character.move(direction,
                           action_factory)
        else:
            character.tick = character.tick + 10

    @logged
    def _combat(self, model, action_factory, rng):
        """
        Attack enemies
        """
        character = self.character
        player = model.player
        c_location = character.location
        p_location = player.location

        distance = ((c_location[0] - p_location[0]) ** 2 +
                    (c_location[1] - p_location[1]) ** 2) ** 0.5

        if distance < 1.42:
            direction = self.find_direction(c_location,
                                            p_location)
            character.perform_attack(direction,
                                     action_factory,
                                     rng)
        else:
            path, connections, updated = a_star(c_location,
                                                p_location,
                                                character.level)
            next_tile = path[1]

            direction = self.find_direction(character.location,
                                            next_tile)

            if character.is_move_legal(direction,
                                       'walk',
                                       action_factory):
                character.move(direction,
                               action_factory)
            else:
                character.tick = character.tick + 10

    @logged
    def find_direction(self, start, end):
        """
        Find direction from start to end

        :param start: start location
        :type start: (integer, integer)
        :param end: end location
        :type end: (integer, integer)

        :returns: Direction to travel
        :rtype: integer
        """
        assert(start != None)
        assert(end != None)

        direction = None
        if start[0] < end[0]:
            #right side
            if start[1] < end[1]:
                #right, below
                direction = 4
            elif start[1] > end[1]:
                #right, above
                direction = 2
            else:
                #right
                direction = 3
        elif start[0] > end[0]:
            #left side
            if start[1] < end[1]:
                #left, below
                direction = 6
            elif start[1] > end[1]:
                #left, above
                direction = 8
            else:
                #left
                direction = 7
        else:
            #up or down
            if start[1] < end[1]:
                #below
                direction = 5
            elif start[1] > end[1]:
                #above
                direction = 1

        return direction



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for Character related classes
"""
from pyherc.aspects import logged
from pyherc.rules import MoveParameters, AttackParameters, DrinkParameters
from pyherc.rules import InventoryParameters, SpellCastingParameters
from pyherc.events import HitPointsChangedEvent, ErrorEvent
from decorator import decorator

@decorator
def guarded_action(wrapped_function, *args, **kwargs):
    """
    Decorator to guard against exceptions in action functions
    """
    try:
        return wrapped_function(*args, **kwargs)
    except Exception:
        self = args[0]
        self.tick = 10
        self.raise_event(ErrorEvent(self))

class Character():
    """
    Represents a character in playing world
    """
    @logged
    def __init__(self, model, effects_collection, inventory):
        """
        Default constructor

        :param model: model where character acts
        :type model: Model
        :param effects_collection: collection for effects
        :type effects_collection: EffectsCollection
        :param inventory: inventory for character
        :type inventory: Inventory
        """
        super(Character, self).__init__()
        # attributes
        self.model = model
        self.__body = None
        self.__finesse = None
        self.__mind = None
        self.name = 'prototype'
        self.race = None
        self.kit = None
        self.__hit_points = None
        self.__max_hp = None
        self.spirit = None
        self.max_spirit = None
        self.speed = None
        self.inventory = inventory
        self.feats = []
        #location
        self.level = None
        self.location = ()
        #icon
        self.icon = None
        #internal
        self.tick = 0
        self.short_term_memory = []
        self.__event_listeners = []
        self.__update_listeners = []
        self.item_memory = {}
        self.size = 'medium'
        self.attack = None
        #mimic
        self.mimic_item = None
        self.__active_effects = [] # active effects
        self.artificial_intelligence = None
        self.__effects_collection = effects_collection

    def __str__(self):
        return self.name

    @logged
    def receive_event(self, event):
        """
        Receives an event from world and enters it into short term memory

        :param event: event to receive
        :type event: Event
        """
        self.short_term_memory.append(event)

        for listener in self.__event_listeners:
            listener.receive_event(event)

    @logged
    def register_event_listener(self, listener):
        """
        Register event listener

        :param listener: listener to add
        :type listener: Listener

        .. versionadded:: 0.4
        """
        self.__event_listeners.append(listener)

    @logged
    def register_for_updates(self, listener):
        """
        Register listener to receive updates for this entity

        :param listener: listener to add
        :type listener: Listener

        .. versionadded:: 0.5
        """
        self.__update_listeners.append(listener)

    @logged
    def remove_from_updates(self, listener):
        """
        Remove listener

        :param listener: listener to remove
        :type listener: Listener

        .. versionadded:: 0.5
        """
        self.__update_listeners.remove(listener)

    @logged
    def notify_update_listeners(self, event):
        """
        Notify all listeners registered for update of this entity

        :param event: event to relay to update listeners
        :type event: Event

        .. versionadded:: 0.5
        """
        for listener in self.__update_listeners:
            listener.receive_update(event)

    @guarded_action
    @logged
    def act(self, model, action_factory, rng):
        """
        Triggers AI of this character

        :param model: model where character is located
        :type model: Model
        :param action_factory: factory for creating actions
        :type action_factory: ActionFactory
        :param rng: random number generator
        :type rng: Random
        """
        self.artificial_intelligence.act(model,
                                         action_factory,
                                         rng)

    def __get_hp(self):
        """
        Current hitpoints
        """
        return self.__hit_points

    def __set_hp(self, hit_points):
        """
        Current hitpoints
        """
        old_hit_points = self.__hit_points
        new_hit_points = hit_points

        self.__hit_points = hit_points

        self.raise_event(
            HitPointsChangedEvent(character = self,
                                  old_hit_points = old_hit_points,
                                  new_hit_points = new_hit_points))

    def __get_body(self):
        """
        Current body attribute
        """
        return self.__body

    def __set_body(self, body):
        """
        Current body attribute
        """
        self.__body = body

    def __get_finesse(self):
        """
        Current finesse attribute
        """
        return self.__finesse

    def __set_finesse(self, finesse):
        """
        Current finesse attribute
        """
        self.__finesse = finesse

    def __get_mind(self):
        """
        Current mind attribute
        """
        return self.__mind

    def __set_mind(self, mind):
        """
        Current mind attribute
        """
        self.__mind = mind

    def get_attack(self):
        """
        Attack attribute of the character
        """
        return self.attack

    def set_attack(self, attack):
        """
        Attack attribute of the character
        """
        self.attack = attack

    def __get_max_hp(self):
        """
        Maximum HP this character can currently have
        """
        return self.__max_hp

    def __set_max_hp(self, max_hp):
        """
        Maximum HP this character can currently have
        """
        self.__max_hp = max_hp

    @logged
    def identify_item(self, item):
        """
        Identify item

        :param item: item to mark as identified
        :type item: Item
        """
        assert (item != None)
        self.item_memory[item.name] = item.name

    @logged
    def is_proficient(self, weapon):
        """
        Check if this character is proficient with a given weapon

        :param weapon: weapon which proficient requirements should be checked
        :type weapon: Item
        :returns: True if proficient, otherwise False
        :rtype: Boolean
        """
        assert weapon != None

        if weapon.weapon_data == None:
            return True

        if True in [(x.name == 'weapon proficiency'
                    and x.weapon_type == weapon.weapon_data.weapon_type)
                    and (x.weapon_name == None
                         or x.weapon_name == weapon.weapon_data.name)
                    for x in self.feats]:
            return True
        else:
            return False

    def set_mimic_item(self, item):
        """
        Sets item this character can mimic or pretend to be

        :param item: item to mimic
        :type item: Item
        """
        self.mimic_item = item

    def get_mimic_item(self):
        """
        Gets item this character can mimic

        :returns: item to mimic
        :rtype: Item
        """
        return self.mimic_item

    def get_location(self):
        """
        Returns location of this character

        :returns: location
        :rtype: (integer, integer)
        """
        return self.location

    def set_location(self, location):
        """
        Sets location of this character

        :param location: location to set
        :type location: (integer, integer)
        """
        self.location = location

    @guarded_action
    @logged
    def execute_action(self, action_parameters, action_factory):
        """
        Execute action defined by action parameters

        :param action_parameters: parameters controlling creation of the action
        :type action_parameters: ActionParameters
        :param action_factory: factory to create actions
        :type action_factory: ActionFactory
        """
        action = self.create_action(action_parameters,
                                    action_factory)
        action.execute()

    @logged
    def create_action(self, action_parameters, action_factory):
        """
        Create an action by defined by action parameters

        :param action_parameters: parameters controlling creation of the action
        :type action_parameters: ActionParameters
        :param action_factory: factory to create actions
        :type action_factory: ActionFactory
        :returns: Action
        """
        action = action_factory.get_action(action_parameters)

        assert action != None

        return action

    @guarded_action
    @logged
    def move(self, direction, action_factory):
        """
        Move this character to specified direction

        :param direction: direction to move
        :type direction: integer
        :param action_factory: factory to create actions
        :type action_factory: ActionFactory
        """
        action = action_factory.get_action(
                                           MoveParameters(
                                                          self,
                                                          direction,
                                                          'walk'))
        action.execute()

    @logged
    def is_move_legal(self, direction, movement_mode, action_factory):
        """
        Check if movement is legal

        :param direction: direction to move
        :type direction: integer
        :param movement_mode: mode of movement
        :type movement_mode: string
        :param action_factory: factory to create actions
        :type action_factory: ActionFactory
        :returns: True if move is legal, False otherwise
        :rtype: Boolean
        """
        action = action_factory.get_action(
                                           MoveParameters(
                                                          self,
                                                          direction,
                                                          movement_mode))
        return action.is_legal()

    @guarded_action
    @logged
    def perform_attack(self, direction, action_factory, rng):
        """
        Attack to given direction

        :param direction: direction to attack
        :type direction: integer
        :param action_factory: factory to create actions
        :type action_factory: ActionFactory
        :param rng: random number generator
        :type rng: Random
        """
        if self.inventory.weapon != None:
            weapon = self.inventory.weapon.weapon_data
            if self.inventory.projectiles != None:
                ammunition = self.inventory.projectiles.ammunition_data
            else:
                ammunition = None
        else:
            weapon = None

        if weapon == None:
            attack_type = 'unarmed'
        else:
            if (ammunition == None or
                    weapon.ammunition_type != ammunition.ammunition_type):
                attack_type = 'melee'
            else:
                target_loc = self.get_location_at_direction(direction)
                if self.level.get_creature_at(target_loc) == None:
                    if self.level.blocks_movement(target_loc[0], target_loc[1]):
                        attack_type = 'melee'
                    else:
                        attack_type = 'ranged'
                else:
                    attack_type = 'melee'

        action = action_factory.get_action(
                            AttackParameters(
                                attacker = self,
                                direction = direction,
                                attack_type = attack_type,
                                random_number_generator = rng))

        action.execute()

    @guarded_action
    @logged
    def drink(self, potion, action_factory):
        """
        Drink potion

        :param potion: potion to drink
        :type potion: Item
        :param action_factory: factory to create actions
        :type action_factory: ActionFactory
        """
        action = action_factory.get_action(
                                           DrinkParameters(
                                                           self,
                                                           potion))
        action.execute()

    @guarded_action
    @logged
    def pick_up(self, item, action_factory):
        """
        Pick up item

        :param item: item to pick up
        :type item: Item
        :param action_factory: factory to create actions
        :type action_factory: ActionFactory

        .. versionadded:: 0.4
        """
        action = action_factory.get_action(
                                    InventoryParameters(
                                                        self,
                                                        item,
                                                        'pick up'))
        action.execute()

    @guarded_action
    @logged
    def drop_item(self, item, action_factory):
        """
        Drop item from inventory

        :param item: item to drop
        :type item: Item
        :param action_factory: factory to create actions
        :type action_factory: ActionFactory

        .. versionadded:: 0.5
        """
        action = action_factory.get_action(
                                InventoryParameters(self,
                                                    item,
                                                    'drop'))
        action.execute()

    @guarded_action
    @logged
    def equip(self, item, action_factory):
        """
        Wear item from inventory

        :param item: item to equip
        :type item: Item
        :param action_factory: factory to create actions
        :type action_factory: ActionFactory

        .. versionadded:: 0.8
        """
        action = action_factory.get_action(
                                InventoryParameters(self,
                                                    item,
                                                    'equip'))
        action.execute()

    @guarded_action
    @logged
    def unequip(self, item, action_factory):
        """
        Unequip item

        :param item: item to unequip
        :type item: Item
        :param action_factory: factory to create actions
        :type action_factory: ActionFactory

        .. versionadded:: 0.8
        """
        action = action_factory.get_action(
                                InventoryParameters(self,
                                                    item,
                                                    'unequip'))
        action.execute()

    @guarded_action
    @logged
    def cast(self, direction, spell_name, action_factory):
        """
        Cast a spell
        
        :param direction: direction to cast the spell
        :type direction: int
        :param spell: spell to cast
        :type spell: string
        :param action_factory: factory to create actions
        :type action_factory: ActionFactory
        
        .. versionadded:: 0.9
        """
        action = action_factory.get_action(
                                SpellCastingParameters(self,
                                                       direction = direction, 
                                                       spell_name = spell_name))
        action.execute()

    @logged
    def raise_event(self, event):
        """
        Raise event for other creatures to see

        :param event: event to raise
        :type event: Event
        """
        self.model.raise_event(event)
        self.notify_update_listeners(event)

    @logged
    def add_effect_handle(self, effect):
        """
        Adds an effect handle to an character

        :param effect: effect to add
        :type effect: EffectHandle

        .. versionadded:: 0.4
        """
        self.__effects_collection.add_effect_handle(effect)

    @logged
    def get_effect_handles(self, trigger = None):
        """
        Get effect handles

        :param trigger: optional trigger type
        :type trigger: string

        :returns: effect handles
        :rtype: [EffectHandle]

        .. versionadded:: 0.4
        """
        return self.__effects_collection.get_effect_handles(trigger)

    @logged
    def remove_effect_handle(self, handle):
        """
        Remove given handle

        :param handle: handle to remove
        :type handle: EffectHandle

        .. versionadded:: 0.4
        """
        self.__effects_collection.remove_effect_handle(handle)

    @logged
    def add_effect(self, effect):
        """
        Adds effect to this character

        :param effect: Effect to add
        :type effect: Effect

        .. note: Multiples of same type of effect are not added
        .. versionadded:: 0.4
        """
        if effect.multiple_allowed == False:
            if not self.__effects_collection.has_effect(effect):
                self.__effects_collection.add_effect(effect)
                self.raise_event(effect.get_add_event())
        else:
            self.__effects_collection.add_effect(effect)
            self.raise_event(effect.get_add_event())

    @logged
    def get_effects(self):
        """
        Get effects of the character

        :returns: effects
        :rtype: [Effect]

        .. versionadded:: 0.4
        """
        return self.__effects_collection.get_effects()

    @logged
    def remove_expired_effects(self):
        """
        Remove all effects that have expired

        .. versionadded:: 0.4
        """
        removed = self.__effects_collection.get_expired_effects()

        if len(removed) > 0:
            self.__effects_collection.remove_expired_effects()
            for effect in removed:
                self.raise_event(effect.get_removal_event())

    def add_to_tick(self, cost):
        """
        Add cost of action to characters tick,
        while taking characters speed into account

        :param cost: Cost of action in ticks
        :type cost: integer
        """
        self.tick = self.tick + (self.speed * cost)

    @logged
    def get_location_at_direction(self, direction):
        """
        Get location next to this character at given direction

        :param direction: direction to check
        :type direction: int
        :returns: coordinates of location
        :rtype: (int, int)

        .. note:: values 1 to 8 are supported for direction
        """
        assert direction <= 8
        assert direction >= 1

        offset = [(0, 0),
                  (0, -1), (1, -1), (1, 0), (1, 1),
                  (0, 1), (-1, 1), (-1, 0), (-1, -1)]

        return tuple(x for x in
                     map(sum, zip(self.location, offset[direction])))

    def _repr_pretty_(self, p, cycle):
        """
        Pretty print for IPython

        :param p: printer to write
        :param cycle: has pretty print detected a cycle?
        """
        if cycle:
            p.text('Character(...)')
        else:
            p.text('name: {0}'.format(self.name))
            p.breakable()
            p.text('hitpoints: {0}/{1}'.format(self.__hit_points,
                                                    self.__max_hp))
            p.breakable()
            p.text('body: {0}'.format(self.__body))
            p.breakable()
            p.text('finesse: {0}'.format(self.__finesse))
            p.breakable()
            p.text('mind: {0}'.format(self.__mind))
            p.breakable()
            p.text('location: {0}'.format(self.location))
            p.breakable()
            p.pretty(self.inventory)
            p.breakable()
            p.pretty(self.__effects_collection)
            p.breakable()

    hit_points = property(__get_hp, __set_hp)
    max_hp = property(__get_max_hp, __set_max_hp)
    body = property(__get_body, __set_body)
    finesse = property(__get_finesse, __set_finesse)
    mind = property(__get_mind, __set_mind)

class Feat():
    """
    Represents a feat that a character can have
    """
    def __init__(self, name = None, target = None):
        self.name = name
        self.target = target

class WeaponProficiency(Feat):
    """
    Represents weapon proficiency feats (proficiency, focus, etc.)
    """
    def __init__(self, weapon_type = 'simple', weapon_name = None):
        Feat.__init__(self, weapon_type, weapon_name)

        self.name = 'weapon proficiency'
        self.weapon_type = weapon_type
        self.weapon_name = weapon_name



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module defining classes related to inventory actions
"""
from pyherc.aspects import logged
from pyherc.rules.factory import SubActionFactory
from pyherc.events import EquipEvent

class EquipFactory(SubActionFactory):
    """
    Factory for creating equip actions

    .. versionadded:: 0.8
    """
    @logged
    def __init__(self):
        """
        Constructor for this factory
        """
        super(EquipFactory, self).__init__()
        self.sub_action = 'equip'

    @logged
    def can_handle(self, parameters):
        """
        Can this factory process these parameters

        :param parameters: parameters to check
        :returns: True if factory is capable of handling parameters
        :rtype: Boolean
        """
        return self.sub_action == parameters.sub_action

    @logged
    def get_action(self, parameters):
        """
        Create an equip action

        :param parameters: parameters used to control creation
        :type parameters: InventoryParameters
        """
        return EquipAction(parameters.character, parameters.item)

class EquipAction():
    """
    Action for equiping an item

    .. versionadded:: 0.8
    """
    @logged
    def __init__(self, character, item):
        """
        Default constructor

        :param character: character wearing the item
        :type character: Character
        :param item: item to equip
        :type item: Item
        """
        super(EquipAction, self).__init__()

        self.character = character
        self.item = item

    @logged
    def execute(self):
        """
        Executes this action
        """
        if self.item.get_main_type() == 'armour':
            self.character.inventory.armour = self.item
            self.character.raise_event(EquipEvent(self.character,
                                                  self.item))
        elif self.item.get_main_type() == 'weapon':
            self.character.inventory.weapon = self.item
            self.character.raise_event(EquipEvent(self.character,
                                                  self.item))
        elif self.item.get_main_type() == 'ammunition':
            self.character.inventory.projectiles = self.item
            self.character.raise_event(EquipEvent(self.character,
                                                  self.item))

    @logged
    def is_legal(self):
        """
        Check if the action is possible to perform

        :returns: True if move is possible, false otherwise
        :rtype: Boolean
        """
        return True



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Classes for item generation
"""
from pyherc.aspects import logged

class ItemAdderConfiguration():
    """
    Configuration for ItemAdder
    """
    @logged
    def __init__(self, level_types):
        """
        Default constructor

        :param level_types: types of level adder can be used at
        :type level_types: [string]
        """
        super(ItemAdderConfiguration, self).__init__()
        self.level_types = level_types
        self.items = []

    @logged
    def add_item(self, min_amount, max_amount, name = None, type = None,
                 location = None):
        """
        Adds item to configuration

        :param min_amount: minimum amount of items to generate
        :type min_amount: integer
        :param max_amount: maximum amount of items to generate
        :type max_amount: integer
        :param name: optional name of item to generate
        :type name: string
        :param type: optional type of item to generate
        :type type: string
        :param location: optional location type for item placement
        :type location: string
        """
        item_spec = {}
        item_spec['min_amount'] = min_amount
        item_spec['max_amount'] = max_amount
        item_spec['name'] = name
        item_spec['type'] = type
        item_spec['location'] = location

        self.items.append(item_spec)


class ItemAdder():
    """
    Class for adding items
    """
    @logged
    def __init__(self, item_generator, configuration, rng):
        """
        Default constructor

        :param item_generator: configured item generator
        :type item_generator: ItemGenerator
        :param configuration: configuration
        :type configuration: ItemAdderConfiguration
        :param rng: random number generator
        :type rng: Random
        """
        super(ItemAdder, self).__init__()
        self.item_generator = item_generator
        self.configuration = configuration
        self.rng = rng

    @logged
    def __get_level_types(self):
        """
        Get level types this item adder can be used at
        """
        return self.configuration.level_types

    @logged
    def add_items(self, level):
        """
        Add items

        :param level: level to add items
        :type level: Level
        """
        item_list = self.configuration.items
        items = []

        for item in item_list:
            items.extend(self.generate_items(item))

        self.place_items(items, level)

    @logged
    def generate_items(self, item_spec):
        """
        Generate items according to specification

        :param item_spec: Dictionary specifying items to create
        :type item_spec: dict
        :returns: tupple (item_spec, item)
        """
        items = []
        amount = self.rng.randint(item_spec['min_amount'],
                                  item_spec['max_amount'])

        for i in range(amount):
            new_item = self.item_generator.generate_item(
                                                name = item_spec['name'],
                                                item_type = item_spec['type'])
            items.append((item_spec, new_item))

        return items

    @logged
    def place_items(self, items, level):
        """
        Place items to level

        :param items: list of tupples (item_spec, item)
        :param level: level to place items
        :type level: Level
        """
        for item in items:
            location_type = item[0]['location']

            if location_type == None:
                location_type = 'any'

            locations = level.get_locations_by_type(location_type)

            location = self.rng.choice(locations)

            level.add_item(item[1], location)

    level_types = property(__get_level_types)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Attack related factories are defined here
"""
from pyherc.aspects import logged
from pyherc.rules.move.action import MoveAction, EscapeAction
from pyherc.rules.factory import SubActionFactory

class WalkFactory(SubActionFactory):
    """
    Factory for creating walk actions
    """
    @logged
    def __init__(self, level_generator_factory):
        """
        Constructor for this factory

        :param level_generator_factory: Factory for generating level generators
        :type level_generator_factory: LevelGeneratorFactory
        """
        self.level_generator_factory = level_generator_factory
        self.movement_mode = 'walk'

    def __str__(self):
        return 'walk factory'

    @logged
    def can_handle(self, parameters):
        """
        Can this factory process these parameters

        :param parameters: Parameters to check
        :type parameters: MoveParameters

        :returns: True if factory is capable of handling parameters
        :rtype: boolean
        """
        return self.movement_mode == parameters.movement_mode

    @logged
    def get_action(self, parameters):
        """
        Create a walk action

        :param parameters: Parameters used to control walk creation
        :type parameters: MoveParameters
        """
        location = parameters.character.location
        new_level = parameters.character.level
        direction = parameters.direction
        new_location = None

        if direction == 1:
            new_location = (location[0], location[1] - 1)
        elif direction == 2:
            new_location = (location[0] + 1, location[1] - 1)
        elif direction == 3:
            new_location = (location[0] + 1, location[1])
        elif direction == 4:
            new_location = (location[0] + 1, location[1] + 1)
        elif direction == 5:
            new_location = (location[0], location[1] + 1)
        elif direction == 6:
            new_location = (location[0] - 1, location[1] + 1)
        elif direction == 7:
            new_location = (location[0] - 1, location[1])
        elif direction == 8:
            new_location = (location[0] - 1, location[1] - 1)
        elif direction == 9:
            portal = new_level.get_portal_at(location)
            if portal != None:
                if not portal.exits_dungeon:
                    other_end = portal.get_other_end(self.level_generator_factory)
                    if other_end != None:
                        new_level = other_end.level
                        new_location = other_end.location
                    else:
                        raise RuntimeError('Portal leads to void!')
                else:
                    return EscapeAction(parameters.character)
            else:
                new_level = parameters.character.level
                new_location = parameters.character.location
        else:
            raise RuntimeError('Character does not know where to go')

        #is new location blocked?
        if new_level.blocks_movement(new_location[0], new_location[1]):
            new_location = parameters.character.location

        return MoveAction(parameters.character, new_location, new_level)

class MoveFactory(SubActionFactory):
    """
    Factory for constructing move actions
    """
    @logged
    def __init__(self, factories):
        """
        Constructor for this factory

        :param factories: a single Factory or list of Factories to use
        """
        self.action_type = 'move'

        if hasattr(factories, '__iter__'):
            self.factories = factories
        else:
            self.factories = []
            self.factories.append(factories)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for Model related classes
"""

from pyherc.aspects import logged

ESCAPED_DUNGEON = 1
DIED_IN_DUNGEON = 2

class Model():
    """
    Represents playing world
    """
    @logged
    def __init__(self):
        """
        Default constructor
        """
        self.dungeon = None
        self.player = None
        self.config = None
        self.tables = None
        self.__event_listeners = []
        self.end_condition = 0

    @logged
    def register_event_listener(self, listener):
        """
        Registers event listener on this model

        :param listener: Listener to register
        """
        assert listener != None
        self.__event_listeners.append(listener)

    @logged
    def get_event_listeners(self):
        """
        Retrieve registered event listeners
        """
        return self.__event_listeners[:]

    @logged
    def raise_event(self, event):
        """
        Relays event to creatures

        :param event: event to relay
        :type event: dict
        """
        level = event.level

        if level != None:
            for creature in level.creatures:
                creature.receive_event(event)

        for listener in self.__event_listeners:
            listener.receive_event(event)

    def get_next_creature(self, rules_engine):
        """
        Get the character who is next to take action

        :param rules_engine: engine containing rules
        :type rules_engine: RulesEngine
        :returns: Character to act next
        :rtype: Character
        """
        level = self.player.level

        if level == None:
            return None

        creatures = level.creatures

        assert len(creatures) != 0
        assert self.player in level.creatures

        while 1:
            for creature in creatures:
                if creature.tick <= 0:
                    return creature

            for creature in creatures:
                creature.tick = creature.tick - 1
                for effect in creature.get_effects():
                    if effect.tick != None:
                        effect.tick = effect.tick - 1
                        if effect.tick == 0:
                            effect.trigger(rules_engine.dying_rules)
                creature.remove_expired_effects()

class Damage():
    """
    Damage done in combat
    """
    def __init__(self, amount = 0, damage_type = 'bludgeoning',
                        magic_bonus = 0):
        self.amount = amount
        self.damage_type = damage_type
        self.magic_bonus = magic_bonus

class MimicData():
    """
    Represents mimicing character
    """
    def __init__(self, character):
        self.fov_matrix = []
        self.character = character

    def get_character(self):
        """
        Get mimicing character

        :returns: Character
        :rtype: Character
        """
        return self.character

    def set_character(self, character):
        """
        Set character mimicing this item

        :param character: Character to set
        :type character: Character
        """
        self.character = character



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module defining classes related to DrinkAction
"""

class DrinkAction():
    """
    Action for drinking
    """
    def __init__(self, character, potion, effect_factory,
                 dying_rules):
        """
        Default constructor

        Args:
            character: Character drinking
            potion: Item to drink
            effect_factory: Initialised EffectsFactory
        """
        self.character = character
        self.potion = potion
        self.effect_factory = effect_factory
        self.dying_rules = dying_rules

    def execute(self):
        """
        Executes this Action
        """
        if self.is_legal():
            self.character.identify_item(self.potion)

            drink_effects = self.potion.get_effect_handles('on drink')

            if len(drink_effects) > 0:
                for effect_spec in drink_effects:
                    effect = self.effect_factory.create_effect(
                                                    effect_spec.effect,
                                                    target = self.character)
                    if effect.duration == 0:
                        effect.trigger(self.dying_rules)
                    else:
                        self.character.add_effect(effect)
                    effect_spec.charges = effect_spec.charges - 1

                if self.potion.maximum_charges_left < 1:
                    self.character.inventory.remove(self.potion)

    def is_legal(self):
        """
        Check if the action is possible to perform

        Returns:
            True if action is possible, false otherwise
        """
        return True



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Classs needed for generating levels
"""

import logging
from pyherc.data import Level, Portal
from pyherc.aspects import logged

class LevelGeneratorFactory():
    """
    Class used to contruct different kinds of level generators
    """
    @logged
    def __init__(self, portal_adder_factory, configuration,
                 random_generator):
        """
        Default constructor

        :param configuration: configuration for factory
        :type configuration: LevelGeneratorFactoryConfiguration
        :param random_generator: random number generator
        :type random_generator: Random
        """
        self.logger = logging.getLogger('pyherc.generators.level.crypt.LevelGeneratorFactory') #pylint: disable=c0301
        self.level_partitioners = configuration.level_partitioners
        self.room_generators = configuration.room_generators
        self.decorators = configuration.decorators
        self.portal_adder_configurations = configuration.portal_adder_configurations
        self.portal_adder_factory = portal_adder_factory
        self.portal_adder_factory.level_generator_factory = self
        self.item_adders = configuration.item_adders
        self.creature_adders = configuration.creature_adders
        self.random_generator = random_generator
        self.level_infos = configuration.contexts

    @logged
    def get_generator(self, level_type):
        """
        Get LevelGenerator for given level

        :param level_type: type of level to generate
        :type level_type: string
        :returns: configured level generator
        :rtype: LevelGenerator
        """
        partitioner = self.get_sub_component(level_type,
                                             self.level_partitioners,
                                             'partitioner')

        rooms = self.get_sub_components(level_type,
                                      self.room_generators,
                                      'room')

        decorator = self.get_sub_component(level_type,
                                           self.decorators,
                                           'decorator')

        item_adder = self.get_sub_component(level_type,
                                            self.item_adders,
                                            'item adder')

        creature_adder = self.get_sub_component(level_type,
                                                self.creature_adders,
                                                'creature adder')

        level_info = self.get_sub_component(level_type,
                                            self.level_infos,
                                            'level info')

        portal_adders = self.portal_adder_factory.create_portal_adders(level_type)

        return LevelGenerator(partitioner,
                              rooms,
                              decorator,
                              portal_adders,
                              item_adder,
                              creature_adder,
                              self.random_generator,
                              level_info)

    @logged
    def get_sub_components(self, level_type, component_list, component_type):
        """
        Get subcomponent

        :param level_type: type of level to generate
        :type level_type: string
        :param component_list: list of subcomponents to choose from
        :type component_list: [object]
        :param component_type: component type for error message
        :type component_type: string
        :returns: components
        :rtype: [object]
        """
        components = [x for x in component_list
                      if level_type in x.level_types]

        if len(components) == 0:
            error_message = "No {0} for type {1} in {2}".format(
                                                            component_type,
                                                            level_type,
                                                            component_list)
            self.logger.error(error_message)
            raise RuntimeError(error_message)

        return components

    @logged
    def get_sub_component(self, level_type, component_list, component_type):
        """
        Get subcomponent

        :param level_type: type of level to generate
        :type level_type: string
        :param component_list: subcomponents to choose from
        :type component_list: [object]
        :param component_type: component type for error message
        :type component_type: string
        :returns: single component
        :rtype: object
        """
        matches = [x for x in component_list
                   if level_type in x.level_types]

        if len(matches) > 0:
            component = self.random_generator.choice(matches)
        else:
            error_message = "No {0} for type {1} in {2}".format(
                                                            component_type,
                                                            level_type,
                                                            component_list)
            self.logger.error(error_message)
            raise RuntimeError(error_message)

        return component

class LevelGenerator():
    """
    Class used to generate levels
    """
    @logged
    def __init__(self, partitioner, room_generators,
                 decorator, portal_adders,
                 item_adder, creature_adder,
                 random_generator, level_context):
        """
        Default constructor

        :param partitioner: LevelPartitioner to use
        :param room_generators: RoomGenerators to use
        :param decorator: LevelDecorator to use
        :param portal_adder: PortalAdder to use
        :param item_adder: ItemAdder to generate items
        :param creature_adder: CreatureAdder to add creatures
        :param random_generator: Random number generator
        :param level_context: Context for level
        :param size: Size of the level to create
        """
        self.logger = logging.getLogger('pyherc.generators.level.crypt.LevelGenerator') #pylint: disable=C0301
        self.item_adder = item_adder
        self.creature_adder = creature_adder
        self.random_generator = random_generator

        self.partitioner = partitioner
        self.room_generators = room_generators
        self.decorator = decorator
        self.portal_adders = portal_adders
        self.level_context = level_context

    @logged
    def generate_level(self, portal):
        """
        Generate level

        :param portal: portal to link to this level
        :type portal: Portal
        """
        new_level = Level(size = self.level_context.size,
                          floor_type = self.level_context.floor_type,
                          wall_type = self.level_context.wall_type,
                          empty_floor = self.level_context.empty_floor,
                          empty_wall = self.level_context.empty_wall)

        sections = self.partitioner.partition_level(new_level)

        for section in sections:
            generator = self.random_generator.choice(self.room_generators)
            generator.generate_room(section)

        self.decorator.decorate_level(new_level)

        for adder in self.portal_adders:
            adder.add_portal(new_level)

        # all this needs to be cleaned up
        if portal != None:
            rooms = new_level.get_locations_by_type('room')
            if len(rooms) > 0:
                new_portal = Portal(icons = (portal.other_end_icon, None),
                                    level_generator_name = None)
                location = self.random_generator.choice(rooms)
                new_level.add_portal(new_portal, location, portal)
            else:
                self.logger.warn('no location found, skipping')


        self.creature_adder.add_creatures(new_level)

        self.item_adder.add_items(new_level)

        self.logger.debug(new_level.dump_string())

        return new_level



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for map screen
"""
from pyherc.aspects import logged
from herculeum.ui.text.inventory import InventoryScreen
from herculeum.ui.text.character import CharacterScreen
from herculeum.ui.text.endscreen import EndScreen
from herculeum.ui.controllers import MoveController, EndScreenController

class MapScreen():
    """
    Class for map screen

    .. versionadded:: 0.9
    """
    @logged
    def __init__(self, model, surface_manager, action_factory, rng,
                 rules_engine, configuration, screen):
        """
        Default constructor
        """
        super(MapScreen, self).__init__()

        self.model = model
        self.surface_manager = surface_manager
        self.action_factory = action_factory
        self.rng = rng
        self.rules_engine = rules_engine
        self.configuration = configuration
        self.screen = screen
        self.keymap, self.move_key_map = self._construct_keymaps(
                                                        configuration.controls)
        self.move_controller = MoveController(action_factory,
                                              rng)
        self.messages = []

    @logged
    def _construct_keymaps(self, config):
        """
        Construct keymaps for handling input
        """
        keymap = {}
        move_keymap = {}
        for key in config.move_left:
            keymap[key] = self._move
            move_keymap[key] = 7
        for key in config.move_up_left:
            keymap[key] = self._move
            move_keymap[key] = 8
        for key in config.move_up:
            keymap[key] = self._move
            move_keymap[key] = 1
        for key in config.move_up_right:
            keymap[key] = self._move
            move_keymap[key] = 2
        for key in config.move_right:
            keymap[key] = self._move
            move_keymap[key] = 3
        for key in config.move_down_right:
            keymap[key] = self._move
            move_keymap[key] = 4
        for key in config.move_down:
            keymap[key] = self._move
            move_keymap[key] = 5
        for key in config.move_down_left:
            keymap[key] = self._move
            move_keymap[key] = 6
        for key in config.action_a:
            keymap[key] = self._action_a

        return keymap, move_keymap


    @logged
    def show(self):
        """
        Show the map
        """
        self.refresh_screen()
        player = self.model.player

        while self.model.end_condition == 0 and player.level != None:
            next_creature = self.model.get_next_creature(self.rules_engine)

            if next_creature == player:
                self._handle_player_input()
            else:
                next_creature.act(model = self.model,
                                  action_factory = self.action_factory,
                                  rng = self.rng)
            self.refresh_screen()

        dialog = EndScreen(model = self.model,
                           dying_rules = self.rules_engine.dying_rules,
                           screen = self.screen,
                           controller = EndScreenController())
        dialog.show()

    @logged
    def _handle_player_input(self):
        """
        Handle input from player
        """
        key = chr(self.screen.getch())

        if key in self.keymap:
            self.keymap[key](key)
        elif key == 'i':
            inv = InventoryScreen(character = self.model.player,
                                  config = self.configuration,
                                  screen = self.screen,
                                  action_factory = self.action_factory,
                                  parent = self.screen)
            inv.show()
        elif key == 'c':
            dialog = CharacterScreen(character = self.model.player,
                                     screen = self.screen)
            dialog.show()
        elif key == 'a':
            dir_key = chr(self.screen.getch())
            if dir_key in self.move_key_map:
                direction = self.move_key_map[dir_key]
                self.model.player.perform_attack(direction,
                                                 self.action_factory,
                                                 self.rng)
        elif key == 'Q':
            self.model.end_condition = 1

    @logged
    def _move(self, key):
        """
        Process movement key

        :param key: key triggering the processing
        :type key: string
        """
        direction = self.move_key_map[key]
        self.move_controller.move_or_attack(self.model.player,
                                            direction,
                                            'walk')
    @logged
    def _action_a(self, key):
        """
        Process action a key

        :param key: key triggering the processing
        :type key: int
        """
        player = self.model.player
        level = player.level
        items = level.get_items_at(player.location)

        if items != None and len(items) > 0:
            player.pick_up(items[0],
                           self.action_factory)

        elif player.is_move_legal(9,
                                  'walk',
                                  self.action_factory):
            player.move(9,
                        self.action_factory)

    @logged
    def refresh_screen(self):
        """
        Draw whole screen
        """
        player = self.model.player
        level = player.level

        if level == None:
            return

        for column_number, column in enumerate(level.floor):
            for row_number, tile in enumerate(column):
                self.screen.addch(row_number + 1,
                                  column_number,
                                  ord(self.surface_manager.get_icon(tile)),
                                  self.surface_manager.get_attribute_by_name('dim'))

        for column_number, column in enumerate(level.walls):
            for row_number, tile in enumerate(column):
                glyph_number = self.surface_manager.get_icon(tile)
                if glyph_number != None:
                    self.screen.addch(row_number + 1,
                                      column_number,
                                      ord(glyph_number),
                                      self.surface_manager.get_attribute_by_name('dim'))

        for portal in level.portals:
            self.screen.addch(portal.location[1] + 1,
                              portal.location[0],
                              ord(self.surface_manager.get_icon(portal.icon)),
                              self.surface_manager.get_attribute(portal.icon))

        for item in level.items:
            self.screen.addch(item.location[1] + 1,
                              item.location[0],
                              ord(self.surface_manager.get_icon(item.icon)),
                              self.surface_manager.get_attribute(item.icon))

        for monster in level.creatures:
            self.screen.addch(monster.location[1] + 1,
                              monster.location[0],
                              ord(self.surface_manager.get_icon(monster.icon)),
                              self.surface_manager.get_attribute(monster.icon))

        self.screen.addch(player.location[1] + 1,
                          player.location[0],
                          ord(self.surface_manager.get_icon(player.icon)),
                          self.surface_manager.get_attribute(player.icon))

        stats = 'HP:{0}({1}) MG:{2}({3})'.format(player.hit_points,
                                                 player.max_hp,
                                                 0,
                                                 0)
        stats = stats.ljust(80)
        self.screen.addstr(22, 0, stats)

        self.screen.refresh()

    @logged
    def receive_event(self, event):
        """
        Receive event
        """
        if event.event_type == 'move':
            self.refresh_screen()

            if event.mover == self.model.player:
                self.messages = []
                self.screen.addstr(0, 0, ' '.ljust(80))

        if event.event_type in ['attack hit',
                                    'attack miss',
                                    'attack nothing',
                                    'poison triggered',
                                    'poison ended',
                                    'poisoned',
                                    'heal started',
                                    'heal ended',
                                    'heal triggered',
                                    'death',
                                    'pick up',
                                    'drop',
                                    'damage triggered',
                                    'equip',
                                    'unequip',
                                    'notice',
                                    'lose focus', 
                                    'error']:
            message = event.get_description(self.model.player)
            self.messages.append(message)
            if len(self.messages) > 2:
                self.messages = self.messages[-2:]
            displayed_messages = ', '.join(self.messages)
            displayed_messages = displayed_messages.ljust(80)
            self.screen.addstr(0, 0, displayed_messages)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration for pyherc
"""
import random
from pyherc.rules.public import ActionFactory
from pyherc.rules import RulesEngine
from pyherc.rules import Dying
from pyherc.rules.move.factories import MoveFactory
from pyherc.rules.move.factories import WalkFactory
from pyherc.rules.consume.factories import DrinkFactory
from pyherc.rules.attack.factories import AttackFactory
from pyherc.rules.attack.factories import UnarmedCombatFactory
from pyherc.rules.attack.factories import MeleeCombatFactory
from pyherc.rules.attack import RangedCombatFactory
from pyherc.rules.inventory.factories import InventoryFactory
from pyherc.rules.inventory.factories import PickUpFactory, DropFactory
from pyherc.rules.inventory.equip import EquipFactory
from pyherc.rules.inventory.unequip import UnEquipFactory
from pyherc.generators import ItemGenerator, CreatureGenerator
from pyherc.generators.level.portals import PortalAdderFactory

from pyherc.generators.level.generator import LevelGeneratorFactory
from pyherc.generators.level.config import LevelGeneratorFactoryConfig

from pyherc.generators import EffectsFactory

from pyherc.generators import ItemConfigurations

class Configuration():
    """
    Configuration object for Herculeum
    """
    def __init__(self, model):
        """
        Default constructor

        Args:
            model: Model to register with factories
        """
        super(Configuration, self).__init__()
        self.resolution = None
        self.full_screen = None
        self.caption = None
        self.action_factory = None
        self.item_generator = None
        self.creature_generator = None
        self.player_generator = None
        self.level_generator_factory = None
        self.level_size = None
        self.model = model
        self.rng = random.Random()
        self.rules_engine = None

    def initialise(self, context):
        """
        Initialises configuration
        """
        self.level_size = (79, 20)

        self.initialise_generators(context)
        self.initialise_level_generators(context)
        self.initialise_factories(context)

    def initialise_factories(self, context):
        """
        Initialises action factory, sub factories and various generators
        """
        dying_rules = Dying()

        walk_factory = WalkFactory(self.level_generator_factory)
        move_factory = MoveFactory(walk_factory)

        effect_factory = EffectsFactory()

        configurators = self.get_configurators(context.config_package,
                                               'init_effects')

        for configurator in configurators:
            effects = configurator(context)
            for effect in effects:
                effect_factory.add_effect(effect[0], effect[1])

        unarmed_combat_factory = UnarmedCombatFactory(effect_factory,
                                                      dying_rules)
        melee_combat_factory = MeleeCombatFactory(effect_factory,
                                                  dying_rules)
        ranged_combat_factory = RangedCombatFactory(effect_factory,
                                                    dying_rules)
        attack_factory = AttackFactory([
                                        unarmed_combat_factory,
                                        melee_combat_factory,
                                        ranged_combat_factory])

        drink_factory = DrinkFactory(effect_factory,
                                     dying_rules)

        inventory_factory = InventoryFactory([PickUpFactory(),
                                             DropFactory(),
                                             EquipFactory(),
                                             UnEquipFactory()])

        self.action_factory = ActionFactory(
                                            self.model,
                                            [move_factory,
                                            attack_factory,
                                            drink_factory,
                                            inventory_factory])

        self.rules_engine = RulesEngine(self.action_factory,
                                        dying_rules)

    def get_creature_config(self, context):
        """
        Load creature configuration

        :param level_config: namespace of configurations
        :type level_config: Package
        :returns: configuration for creatures
        :rtype: CreatureConfigurations
        """
        config = {}

        configurators = self.get_configurators(context.config_package,
                                               'init_creatures')

        for configurator in configurators:
            creatures = configurator(context)
            for creature in creatures:
                config[creature.name] = creature

        return config

    def get_player_config(self, context):
        """
        Load player configuration

        :param level_config: namespace of configurations
        :type level_config: Package
        :returns: configuration for player characters
        :rtype: CreatureConfigurations

        .. versionadded:: 0.8
        """
        config = {}

        configurators = self.get_configurators(context.config_package,
                                               'init_players')

        for configurator in configurators:
            creatures = configurator(context)
            for creature in creatures:
                config[creature.name] = creature

        return config

    def get_item_config(self, context):
        """
        Load item configuration

        :param level_config: namespace of configurations
        :type level_config: Package
        :returns: configuration for items
        :rtype: ItemConfigurations
        """
        config = ItemConfigurations(self.rng)

        configurators = self.get_configurators(context.config_package,
                                               'init_items')

        for configurator in configurators:
            items = configurator(context)
            for item in items:
                config.add_item(item)

        return config

    def get_configurators(self, location, function_name):
        """
        Get functions to configure given sub-system
        """

        config_modules = map(lambda x: getattr(location, x),
                             filter(lambda x: x[0] != '_',
                                    dir(location)))
        configurators = map(lambda x: getattr(x, function_name),
                            filter(lambda y: hasattr(y, function_name),
                                   config_modules))

        return configurators

    def initialise_generators(self, context):
        """
        Initialise generators
        """
        self.item_generator = ItemGenerator(
                                    self.get_item_config(context))

        self.creature_generator = CreatureGenerator(
                                        self.get_creature_config(context),
                                        self.model,
                                        self.item_generator,
                                        self.rng)

        self.player_generator = CreatureGenerator(
                                        self.get_player_config(context),
                                        self.model,
                                        self.item_generator,
                                        self.rng)

    def initialise_level_generators(self, context):
        """
        Initialise level generators

        :param level_config: module containing level configurations
        :type level_config: module
        """
        config = LevelGeneratorFactoryConfig([],
                                             [],
                                             [],
                                             [],
                                             [],
                                             [],
                                             [])

        configurators = self.get_configurators(context.config_package,
                                               'init_level')

        for configurator in configurators:
            self.extend_configuration(config,
                                      configurator(self.rng,
                                                   self.item_generator,
                                                   self.creature_generator,
                                                   self.level_size,
                                                   context))

        portal_adder_factory = PortalAdderFactory(
                                config.portal_adder_configurations,
                                self.rng)

        self.level_generator_factory = LevelGeneratorFactory(
                                                    portal_adder_factory,
                                                    config,
                                                    self.rng)

    def extend_configuration(self, config, new_config):
        """
        Sums two configurations together

        Args:
            config: config to extend
            new_config: items to add to configuration
        """
        config.room_generators.extend(new_config.room_generators)
        config.level_partitioners.extend(new_config.level_partitioners)
        config.decorators.extend(new_config.decorators)
        config.item_adders.extend(new_config.item_adders)
        config.creature_adders.extend(new_config.creature_adders)
        config.portal_adder_configurations.extend(
                                    new_config.portal_adder_configurations)
        config.contexts.extend(new_config.contexts)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for ranged combat
"""
from pyherc.aspects import logged
from pyherc.rules.attack.action import ToHit, Damage, AttackAction

class RangedToHit(ToHit):
    """
    Class to perform to hit calculations in ranged combat

    .. versionadded:: 0.8
    """

    @logged
    def __init__(self, attacker, target, rng):
        """
        Default constructor

        :param attacker: character attacking
        :type attacker: Character
        :param target: target of the attack
        :type target: Character
        :rng: random number generator
        """
        self.attacker = attacker
        self.target = target
        self.rng = rng

class RangedDamage(Damage):
    """
    Damage done with ranged attack

    .. versionadded:: 0.8
    """

    @logged
    def __init__(self, damage):
        """
        Default constructor
        """
        super(RangedDamage, self).__init__(damage)

class RangedCombatFactory():
    """
    Factory for producing ranged combat actions

    .. versionadded:: 0.8
    """
    @logged
    def __init__(self, effect_factory, dying_rules):
        """
        Constructor for this factory
        """
        self.attack_type = 'ranged'
        self.effect_factory = effect_factory
        self.dying_rules = dying_rules

    @logged
    def can_handle(self, parameters):
        """
        Can this factory process these parameters

        :param parameters: parameters to check
        :type parameters: AttackParameters
        :returns: true if factory is capable of handling parameters
        :rtype: boolean
        """
        return self.attack_type == parameters.attack_type

    @logged
    def get_action(self, parameters):
        """
        Create a attack action

        :param parameters: parameters used to control attack creation
        :type parameters: AttackParameters
        :returns: action that can be executed
        :rtype: AttackAction
        """
        attacker = parameters.attacker
        target = self.get_target(parameters)
        weapon = attacker.inventory.projectiles
        ammo_data = weapon.ammunition_data

        attack = AttackAction(
                    attack_type = 'ranged',
                    to_hit = RangedToHit(attacker, target,
                                         parameters.random_number_generator),
                    damage = RangedDamage(damage = ammo_data.damage),
                    attacker = attacker,
                    target = target,
                    effect_factory = self.effect_factory,
                    dying_rules = self.dying_rules,
                    additional_rules = AdditionalRangedRules(attacker))

        return attack

    @logged
    def get_target(self, parameters):
        """
        Get target of the attack

        :param parameters: parameters to control attack
        :type parameters: MeleeAttackParameters
        :returns: target character if found, otherwise None
        :rtype: Character
        """
        location = parameters.attacker.location
        level = parameters.attacker.level
        direction = parameters.direction
        target = None
        off_sets = [(0, 0),
                    (0, -1), (1, -1), (1, 0), (1, 1),
                    (0, 1), (-1, 1), (-1, 0), (-1, -1)]

        while target == None and not level.blocks_movement(location[0], location[1]):
            location = tuple([x for x in
                              map(sum, zip(location, off_sets[direction]))])
            target = level.get_creature_at(location)

        return target

class AdditionalRangedRules():
    """
    Additional rules for ranged attack

    .. versionadded:: 0.8
    """
    def __init__(self, attacker):
        """
        Default constructor
        """
        super(AdditionalRangedRules, self).__init__()

        self.attacker = attacker

    def after_attack(self):
        """
        Processing happening after an attack
        """
        ammunition = self.attacker.inventory.projectiles
        ammunition.ammunition_data.count = ammunition.ammunition_data.count - 1

        if ammunition.ammunition_data.count <= 0:
            self.attacker.inventory.projectiles = None
            self.attacker.inventory.remove(ammunition)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for poison
"""
from pyherc.aspects import logged
from pyherc.events import PoisonTriggeredEvent, PoisonAddedEvent
from pyherc.events import PoisonEndedEvent
from pyherc.data.effects.effect import Effect

class Poison(Effect):
    """
    Class representing effects of poison
    """
    @logged
    def __init__(self, duration, frequency, tick, damage, target,
                 icon, title, description):
        """
        Default constructor
        """
        super(Poison, self).__init__(duration = duration,
                                     frequency = frequency,
                                     tick = tick,
                                     icon = icon,
                                     title = title,
                                     description = description)
        self.damage = damage
        self.target = target
        self.effect_name = 'poison'

    @logged
    def do_trigger(self, dying_rules):
        """
        Triggers effects of the poison
        """
        self.target.hit_points = self.target.hit_points - self.damage

        self.target.raise_event(
                        PoisonTriggeredEvent(target = self.target,
                                             damage = self.damage))

        dying_rules.check_dying(self.target)

    @logged
    def get_add_event(self):
        """
        Get event describing adding of this effect

        :returns: event describing adding of this effect
        :rtype: Event
        """
        return PoisonAddedEvent(target = self.target,
                                effect = self)

    @logged
    def get_removal_event(self):
        """
        Get event describing removal of this event

        :return: event describing removal of this event
        :rtype: Event
        """
        return PoisonEndedEvent(target = self.target,
                                effect = self)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for UI Configuration
"""
from PyQt4.QtCore import Qt

class QtControlsConfiguration():
    """
    Configuration for user interface controls

    .. versionadded:: 0.8
    """
    def __init__(self):
        """
        Default constructor
        """
        super(QtControlsConfiguration, self).__init__()

        self.move_left = [Qt.Key_4, Qt.Key_Left, Qt.Key_H]
        self.move_up_left = [Qt.Key_7, Qt.Key_Y]
        self.move_up = [Qt.Key_8, Qt.Key_Up, Qt.Key_K]
        self.move_up_right = [Qt.Key_9, Qt.Key_U]
        self.move_right = [Qt.Key_6, Qt.Key_Right, Qt.Key_L]
        self.move_down_right = [Qt.Key_3, Qt.Key_N]
        self.move_down = [Qt.Key_2, Qt.Key_Down, Qt.Key_J]
        self.move_down_left = [Qt.Key_1, Qt.Key_B]

        self.action_a = [Qt.Key_Space, Qt.Key_Z, Qt.Key_5]
        self.action_b = [Qt.Key_Control, Qt.Key_X]
        self.action_x = [Qt.Key_Alt]
        self.action_y = [Qt.Key_Tab]

        self.start = [Qt.Key_Return]
        self.back = [Qt.Key_Escape]

        self.left_shoulder = [Qt.Key_Insert, Qt.Key_PageUp, Qt.Key_Q]
        self.right_shoulder = [Qt.Key_Delete, Qt.Key_PageDown, Qt.Key_W]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Package for rules of the game
"""
from .public import ActionFactory
from .public import ActionParameters
from .public import AttackParameters
from .public import MoveParameters
from .public import DrinkParameters
from .public import InventoryParameters
from .public import SpellCastingParameters
from .ending import Dying
from .engine import RulesEngine



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module containing classes to represent Portals
"""

from pyherc.aspects import logged

class Portal():
    """
    Portal linking two levels together
    """
    @logged
    def __init__(self, icons, level_generator_name):
        """
        Default constructor

        :param icons: (my_icon, icon for other end)
        :type icons: (integer, integer)
        :param level_generator_name: name of level generator for proxy portals
        :type level_generator_name: String
        """
        super(Portal, self).__init__()
        self.level = None
        self.location = ()
        self.__icons = icons
        self.__other_end = None
        self.exits_dungeon = False
        self.level_generator_name = level_generator_name
        self.model = None
        self.__update_listeners = []

    @logged
    def get_other_end(self, level_generator_factory):
        """
        Returns the other end of the portal

        :param level_generator_factory: factory to generate level generators
        :type level_generator_factory: LevelGeneratorFactory
        :returns: other end of the portal
        :rtype: Portal
        """
        if self.__other_end == None:
            level_generator = level_generator_factory.get_generator(
                                                    self.level_generator_name)
            level_generator.generate_level(self)

        return self.__other_end

    @logged
    def set_other_end(self, portal):
        """
        Set the other end of the portal

        :param portal: portal where this one leads
        :type portal: Portal
        """
        self.__other_end = portal

    def __get_icon(self):
        """
        Get icon to display this portal

        :returns: icon of the portal
        :rtype: integer
        """
        return self.__icons[0]

    def __set_icon(self, icon):
        """
        Set icon to display this portal

        :param icon: icon to use for the portal
        :type icon: integer
        """
        if self.__icons == None:
            self.__icons = (None, None)
        self.__icons = (icon, self.__icons[1])

    def __get_other_end_icon(self):
        """
        Get icon used for other end of this portal

        :returns: icon of the other end
        :rtype: integer
        """
        return self.__icons[1]

    @logged
    def register_for_updates(self, listener):
        """
        Register listener to receive updates for this entity

        :param listener: listener to add
        :type listener: Listener

        .. versionadded:: 0.5
        """
        self.__update_listeners.append(listener)

    @logged
    def remove_from_updates(self, listener):
        """
        Remove listener

        :param listener: listener to remove
        :type listener: Listener

        .. versionadded:: 0.5
        """
        self.__update_listeners.remove(listener)

    @logged
    def notify_update_listeners(self, event):
        """
        Notify all listeners registered for update of this entity

        :param event: event to relay to update listeners
        :type event: Event

        .. versionadded:: 0.5
        """
        for listener in self.__update_listeners:
            listener.receive_update(event)

    icon = property(__get_icon, __set_icon)
    other_end_icon = property(__get_other_end_icon)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for spell objects
"""

from pyherc.data.effects import EffectsCollection
from pyherc.aspects import logged

class Spell():
    """
    Class to represent spells
    
    .. versionadded:: 0.9
    """
    
    @logged
    def __init__(self):
        """
        Default constructor
        """
        self.targets = []
        self.effects = EffectsCollection()

    @logged
    def add_effect_handle(self, handle):
        """
        Add effect handle

        :param handle: effect handle to add
        :type handle: EffectHandle
        """
        self.effects.add_effect_handle(handle)

    @logged
    def get_effect_handles(self, trigger = None):
        """
        Get effect handles

        :param trigger: optional trigger type
        :type trigger: string

        :returns: effect handles
        :rtype: [EffectHandle]
        """
        return self.effects.get_effect_handles(trigger)

    @logged
    def remove_effect_handle(self, handle):
        """
        Remove given handle

        :param handle: handle to remove
        :type handle: EffectHandle
        """
        self.effects.remove_effect_handle(handle)

    @logged
    def cast(self, effects_factory, dying_rules):
        """
        Cast the spell

        :param effects_factory: factory for creating effects
        :type effects_factory: EffectsFactory
        :param dying_rules: rules for dying
        :type dying_rules: Dying
        """
        handles = self.effects.get_effect_handles('on spell hit')
        effects = []

        for target in self.targets:
            for handle in handles:
                effects.append(effects_factory.create_effect(
                                                key = handle.effect,
                                                target = target))

        for effect in effects:
            effect.trigger(dying_rules)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Classes for events
"""
class Event():
    """
    Super class for events

    .. versionadded:: 0.4
    """
    def __init__(self, event_type, level, location, affected_tiles):
        """
        Default constructor

        :param event_type: type of event
        :type event_type: string
        :param level: level where event happened
        :type level: Level
        :param location: location where event happened
        :type location: (int, int)
        :param affected_tiles: tiles affected by the event
        :type affected_tiles: [(int, int)]
        """
        super(Event, self).__init__()
        self.__level = level
        self.__location = location
        self.__affected_tiles = affected_tiles
        self.__event_type = event_type

    def __get_event_type(self):
        """
        Type of the event

        :rtype: string
        """
        return self.__event_type

    def __get_affected_tiles(self):
        """
        Tiles affected by this event

        :rtype: [(int, int)]
        """
        return self.__affected_tiles

    def __get_location(self):
        """
        Location of the event

        :rtype: (int, int)
        """
        return self.__location

    def __get_level(self):
        """
        Level where event was triggered

        :rtype: Level
        """
        return self.__level

    event_type = property(__get_event_type)
    affected_tiles = property(__get_affected_tiles)
    location = property(__get_location)
    level = property(__get_level)

    def get_description(self, point_of_view):
        """
        Description of the event

        :param point_of_view: point of view for description
        :type point_of_view: Character
        :returns: description of the event
        :rtype: string
        """
        return 'I do something'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for creature generation related classes
"""
from pyherc.data import Character, Inventory
from pyherc.data.effects import EffectHandle, EffectsCollection
from pyherc.aspects import logged

class CreatureGenerator():
    """
    Class used to generate creatures
    """
    @logged
    def __init__(self, configuration, model, item_generator, rng):
        """
        Default constructor
        """
        super(CreatureGenerator, self).__init__()
        self.configuration = configuration
        self.model = model
        self.rng = rng
        self.item_generator = item_generator

    @logged
    def generate_creature(self, name):
        """
        Generate creature

        :param name: name of the creature to generate
        :type name: string
        """
        config = self.__get_creature_config(name)

        new_creature = Character(self.model,
                                 EffectsCollection(),
                                 Inventory())

        new_creature.name = config.name
        new_creature.body = config.body
        new_creature.finesse = config.finesse
        new_creature.mind = config.mind
        new_creature.hit_points = config.hp
        new_creature.max_hp = config.hp
        new_creature.speed = config.speed
        new_creature.icon = config.icons #TODO: pick random
        new_creature.attack = config.attack

        for spec in config.effect_handles:
            new_handle = EffectHandle(trigger = spec.trigger,
                                       effect = spec.effect,
                                       parameters = spec.parameters,
                                       charges = spec.charges)

            new_creature.add_effect_handle(new_handle)

        for spec in config.effects:
            new_effect = spec.clone()
            new_creature.add_effect(new_effect)


        for spec in config.inventory:
            new_creature.inventory.append(
                                self.item_generator.generate_item(
                                                        name = spec.item_name))

        if not config.ai == None:
            new_creature.artificial_intelligence = config.ai(new_creature)

        return new_creature

    @logged
    def __get_creature_config(self, name):
        """
        Get creature config

        :param name: name of the creature
        :type name: string
        """
        return self.configuration[name]

class CreatureConfiguration():
    """
    Configuration for an creature
    """

    def __init__(self, name, body, finesse, mind, hp, speed, icons, attack,
                 ai = None, effect_handles = None, effects = None,
                 inventory = None, description = None):
        """
        Default constructor
        """
        self.name = name
        self.body = body
        self.finesse = finesse
        self.mind = mind
        self.hp = hp
        self.speed = speed
        self.icons = icons
        self.attack = attack
        self.ai = ai
        self.description = description

        if effect_handles == None:
            self.effect_handles = []
        else:
            self.effect_handles = effect_handles

        if effects == None:
            self.effects = []
        else:
            self.effects = effects

        if inventory == None:
            self.inventory = []
        else:
            self.inventory = inventory

class InventoryConfiguration():
    """
    Configuration for inventory of a creature

    .. versionadded:: 0.6
    """
    def __init__(self, item_name, min_amount, max_amount, probability):
        """
        Default constructor
        """
        self.item_name = item_name
        self.min_amount = min_amount
        self.max_amount = max_amount
        self.probability = probability



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for displaying menu
"""
from herculeum.ui.gui.inventory import InventoryWidget
from herculeum.ui.gui.character import CharacterWidget
from PyQt4.QtGui import QDialog, QTabWidget, QVBoxLayout

class MenuDialog(QDialog):
    """
    Dialog to show menu

    .. versionadded:: 0.7
    """
    def __init__(self, surface_manager, character, action_factory, config,
                 parent, flags):
        """
        Default constructor
        """
        super(MenuDialog, self).__init__(parent, flags)

        self.__set_layout(surface_manager,
                          character,
                          action_factory,
                          config,
                          parent)

    def __set_layout(self, surface_manager, character, action_factory,
                     config, parent):
        """
        Set layout of this widget
        """
        self.keymap = self._construct_keymap(config)
        self.setWindowTitle('Menu')
        self.inventory = InventoryWidget(surface_manager = surface_manager,
                                         character = character,
                                         action_factory = action_factory,
                                         config = config,
                                         parent = parent)

        self.character = CharacterWidget(surface_manager = surface_manager,
                                         character = character,
                                         parent = parent)

        self.tabs = QTabWidget()
        self.tabs.addTab(self.inventory, 'Inventory')
        self.tabs.addTab(self.character, 'Character')

        layout = QVBoxLayout()
        layout.addWidget(self.tabs)

        self.setLayout(layout)
        self.inventory.items_carried.items[0].setFocus()

    def _construct_keymap(self, config):
        """
        Construct key map

        .. versionadded:: 0.8
        """
        keymap = {}

        for key in config.left_shoulder:
            keymap[key] = self._switch_left
        for key in config.right_shoulder:
            keymap[key] = self._switch_right
        for key in config.start:
            keymap[key] = self._menu

        return keymap

    def _switch_left(self):
        """
        Switch page to left

        .. versionadded:: 0.8
        """
        current_tab = self.tabs.currentIndex()
        if current_tab > 0:
            self.tabs.setCurrentIndex(current_tab - 1)

    def _switch_right(self):
        """
        Switch page to right

        .. versionadded:: 0.8
        """
        current_tab = self.tabs.currentIndex()
        if current_tab < self.tabs.count():
            self.tabs.setCurrentIndex(current_tab + 1)

    def _menu(self):
        """
        Process menu key

        .. versionadded:: 0.8
        """
        self.done(0)

    def keyPressEvent(self, event):
        """
        Handle keyboard events
        """
        key = event.key()

        if key in self.keymap:
            self.keymap[key]()
        else:
            super(MenuDialog, self).keyPressEvent(event)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for handling loading of images and icons
"""

from pyherc.aspects import logged
from PyQt4.QtGui import QPixmap
from PyQt4.QtCore import QRect
import herculeum.ui.gui.resources

class QtSurfaceManager():
    """
    Class for managing images and icons
    """
    @logged
    def __init__(self):
        """
        Default constructor
        """
        super(QtSurfaceManager, self).__init__()
        self.icons = {}
        self.images = {}
        self.resourcesLoaded = 0

    def __load_image(self, image_name):
        """
        Load a file and return corresponding surface object

        :param image_name: file name
        :type image_name: string

        :returns: image
        :rtype: QPixmap
        """
        image = QPixmap()
        image.load(image_name)

        return image

    @logged
    def add_icon(self, key, filename, ascii_char, attributes = None):
        """
        Add icon to internal collection
        """
        if not key in self.icons:
            surface = self.__load_image(filename)
            self.icons[key] = surface

        return key

    @logged
    def load_resources(self):
        """
        Load graphics from files

        """
        surface = self.__load_image(':transparent.png')
        self.icons['transparent'] = surface

    @logged
    def split_surface(self, image, tile_size):
        """
        Split image to tiles

        :param image: image to split
        :type image: QPixmap
        :param tile_size: size of the tile
        :type tile_size: (int, int)
        :returns: array of tiles
        :rtype: [QPixmap]
        """
        tiles = []
        image_width = image.width()
        image_height = image.height()

        tile_width = tile_size[0]
        tile_height = tile_size[1]

        x_repeats = image_width / tile_width
        y_repeats = image_height / tile_height

        for loc_y in range(y_repeats):
            for loc_x in range(x_repeats):
                rect = QRect(loc_x * tile_width,
                             loc_y * tile_height,
                             tile_width,
                             tile_height)

                sub_image = image.copy(rect)
                tiles.append(sub_image)

        return tiles

    @logged
    def get_image(self, id):
        """
        Get image with ID

        :param id: ID number of the image to retrieve
        :type id: int
        :returns: image
        :rtype: QPixmap
        """
        return self.images[id]

    @logged
    def get_icon(self, id):
        """
        Get icon with ID

        :param id: ID number of the icon to retrieve
        :type id: int
        :returns: icon if found, otherwise empty icon
        :rtype: QPixmap
        """
        if id in self.icons:
            return self.icons[id]
        else:
            print('unknown id: {0}'.format(id))
            return self.icons['transparent']



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for proto tile IDs

These IDs are used to mark tiles on generated levels before the level has been
processed by decorator. This allows level generators to work on higher level of
abstraction and real theme of the level can be changed by using different
decorators.
"""

FLOOR_EMPTY = -1
FLOOR_NATURAL = -2
FLOOR_CONSTRUCTED = -3

WALL_EMPTY = -100
WALL_NATURAL = -101
WALL_CONSTRUCTED = -102



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for aspects
"""
import logging, sys
from decorator import decorator

@decorator
def info_logger(wrapped_function, *args, **kwargs):
    """
    Decorator to perform logging
    """
    logger_name = str(wrapped_function)
    logger = logging.getLogger(logger_name)

    call_message = ' '.join([logger_name,
                             'call',
                             ':',
                             str(args),
                             str(kwargs)])

    logger.debug(call_message)
    
    try:
        result = wrapped_function(*args, **kwargs)

        result_message = ' '.join([logger_name,
                                   'return',
                                   ':',
                                   str(result)])

        logger.debug(result_message)

        return result
    except Exception as err:
        logger.critical(err)
        raise

@decorator
def fatal_logger(wrapped_function, *args, **kwargs):
    """
    Decorator to perform logging of exceptions
    """
    
    try:
        return wrapped_function(*args, **kwargs)

    except Exception as err:
        
        logger_name = str(wrapped_function)
        logger = logging.getLogger(logger_name)

        call_message = ' '.join([logger_name,
                                 'call',
                                 ':',
                                 str(args),
                                 str(kwargs)])
        logger.debug(call_message)        
        logger.critical(err)
        raise

def no_logger(wrapped_function):
    """
    Decorator to perform no logging at all
    """
    
    return wrapped_function



log_levels = {'no_log': no_logger,
              'error_log': fatal_logger,
              'full_log': info_logger}

logged = info_logger

args = sys.argv
for argument in args:
    if argument in log_levels:
        logged = log_levels[argument]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for moving
"""
class MoveController():
    """
    Controller for moving around

    .. versionadded:: 0.9
    """
    def __init__(self, action_factory, rng):
        """
        Default constructor
        """
        super(MoveController, self).__init__()

        self.action_factory = action_factory
        self.rng = rng

    def move_or_attack(self, character, direction, movement_mode):
        """
        Move or attack
        """
        level = character.level

        if character.is_move_legal(direction,
                                   movement_mode,
                                   self.action_factory):
            character.move(direction,
                           self.action_factory)
        elif direction != 9:
            loc = character.get_location_at_direction(direction)
            if level.get_creature_at(loc) != None:
                character.perform_attack(direction,
                                         self.action_factory,
                                         self.rng)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Classes for generating catacombs
"""
from pyherc.aspects import logged
from pyherc.generators.utils import BSPSection

class CatacombsGenerator():
    """
    Class for generating a catacomblike rooms
    """
    @logged
    def __init__(self, floor_tile, empty_tile, level_types, rng):
        """
        Default constructor

        :param floor_tile: id of the tile to use for floors
        :type floor_tile: integer
        :param empty_tile: id of the empty wall tile
        :type empty_tile: integer
        :param level_types: types of level this generator can be used
        :type level_types: [string]
        :param rng: random number generator
        :type rng: Random
        """
        self.floor_tile = floor_tile
        self.empty_tile = empty_tile
        self.room_width = None
        self.room_height = None
        self.level_types = level_types
        self.rng = rng

    @logged
    def generate_room(self, section):
        """
        Generate room

        :param section: section for generator to draw to
        :type section: Section
        """
        level_size = (section.width, section.height)
        room_min_size = (3, 3)
        BSPStack = []
        BSP = BSPSection((0, 0),
                            (level_size[0] - 2,
                            level_size[1] - 2), None)
        BSPStack.append(BSP)
        room_stack = []

        while len(BSPStack) > 0:
            tempBSP = BSPStack.pop()
            tempBSP.split(min_size = (room_min_size[0] + 4,
                                      room_min_size[1] + 4))
            if tempBSP.node1 != None:
                BSPStack.append(tempBSP.node1)
            if tempBSP.node2 != None:
                BSPStack.append(tempBSP.node2)
            if tempBSP.node1 == None and tempBSP.node2 == None:
                #leaf
                room_stack.append(tempBSP)

        for room in room_stack:
            corner1 = (room.corner1[0] + self.rng.randint(1, 4),
                       room.corner1[1] + self.rng.randint(1, 4))
            corner2 = (room.corner2[0] - self.rng.randint(1, 4),
                       room.corner2[1] - self.rng.randint(1, 4))

            for y in range(corner1[1], corner2[1] + 1):
                for x in range(corner1[0], corner2[0] + 1):
                    section.set_floor((x, y),
                                       self.floor_tile,
                                       'room')
                    section.set_wall((x, y),
                                      self.empty_tile,
                                      None)

        area_queue = BSP.get_area_queue()
        area_queue.reverse()

        while len(area_queue) > 1:
            area1 = area_queue.pop()
            area2 = area_queue.pop()
            center1 = area1.get_center()
            center2 = area2.get_center()
            #connect these two areas
            if area1.direction == 1:
                #areas on top of each other
                if center1[1] < center2[1]:
                    for y in range(center1[1], center2[1] + 1):
                        section.set_floor((center1[0], y),
                                          self.floor_tile,
                                          'corridor')
                        section.set_wall((center1[0], y),
                                          self.empty_tile,
                                          None)
                else:
                    for y in range(center2[1], center1[1] + 1):
                        section.set_floor((center1[0], y),
                                          self.floor_tile,
                                          'corridor')
                        section.set_wall((center1[0], y),
                                          self.empty_tile,
                                          None)
            else:
                #areas next to each other
                if center1[0] < center2[0]:
                    for x in range(center1[0], center2[0] + 1):
                        section.set_floor((x, center1[1]),
                                          self.floor_tile,
                                          'corridor')
                        section.set_wall((x, center1[1]),
                                          self.empty_tile,
                                          None)
                else:
                    for x in range(center2[0], center1[0] + 1):
                        section.set_floor((x, center1[1]),
                                          self.floor_tile,
                                          'corridor')
                        section.set_wall((x, center1[1]),
                                          self.empty_tile,
                                          None)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Inventory manipulation related factories are defined here
"""
from pyherc.rules.inventory.action import PickUpAction, DropAction
from pyherc.rules.factory import SubActionFactory
from pyherc.aspects import logged

class PickUpFactory(SubActionFactory):
    """
    Factory for creating pick up actions

    .. versionadded:: 0.4
    """
    @logged
    def __init__(self):
        """
        Constructor for this factory
        """
        super(PickUpFactory, self).__init__()
        self.sub_action = 'pick up'

    @logged
    def can_handle(self, parameters):
        """
        Can this factory process these parameters

        :param parameters: parameters to check
        :returns: True if factory is capable of handling parameters
        :rtype: Boolean
        """
        return self.sub_action == parameters.sub_action

    @logged
    def get_action(self, parameters):
        """
        Create a pick up action

        :param parameters: parameters used to control creation
        :type parameters: InventoryParameters
        """
        return PickUpAction(parameters.character, parameters.item)

class DropFactory(SubActionFactory):
    """
    Factory for creating drop actions

    .. versionadded:: 0.5
    """
    @logged
    def __init__(self):
        """
        Constructor for this factory
        """
        super(DropFactory, self).__init__()
        self.sub_action = 'drop'

    @logged
    def can_handle(self, parameters):
        """
        Can this factory process these parameters

        :param parameters: parameters to check
        :returns: True if factory is capable of handling parameters
        :rtype: Boolean
        """
        return self.sub_action == parameters.sub_action

    @logged
    def get_action(self, parameters):
        """
        Create a drop action

        :param parameters: parameters used to control creation
        :type parameters: InventoryParameters
        """
        return DropAction(parameters.character, parameters.item)

class InventoryFactory(SubActionFactory):
    """
    Factory for constructing inventory actions

    .. versionadded:: 0.4
    """
    @logged
    def __init__(self, factories):
        """
        Constructor for this factory

        :param factories: a single Factory or list of Factories to use
        :type factories: SubActionFactory or [SubActionFactory]
        """
        super(InventoryFactory, self).__init__()
        self.action_type = 'inventory'

        if hasattr(factories, '__iter__'):
            self.factories = factories
        else:
            self.factories = []
            self.factories.append(factories)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for main window related functionality
"""
from herculeum.ui.text.start_game import StartGameScreen
from herculeum.ui.text.map import MapScreen

class MainWindow():
    """
    Main window of the interface

    .. versionadded:: 0.9
    """
    def __init__(self, application, surface_manager, screen,
                 controller):
        """
        Default constructor
        """
        super(MainWindow, self).__init__()

        self.application = application
        self.surface_manager = surface_manager
        self.screen = screen
        self.controller = controller

    def show_new_game(self):
        """
        Show new game dialog
        """
        start_screen = StartGameScreen(self.application.player_generator,
                                       self.application,
                                       self.screen)

        player = start_screen.show()

        self.controller.setup_world(self.application.world,
                                    player)

    def show_map_window(self):
        """
        Show window for playing
        """
        map = MapScreen(model = self.application.world,
                        surface_manager = self.application.surface_manager,
                        action_factory = self.application.action_factory,
                        rng = self.application.rng,
                        rules_engine = self.application.rules_engine,
                        configuration = self.application.config,
                        screen = self.screen)

        self.application.world.register_event_listener(map)
        map.show()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
module for configuring effects
"""
from pyherc.data.effects import Heal, Poison, Damage

def init_effects(context):
    """
    Initialise common effects

    :returns: effect configuration
    """
    surface_manager = context.surface_manager
    config = []

    config.append(('cure minor wounds',
                   {'type': Heal,
                    'duration': 20,
                    'frequency': 5,
                    'tick': 2,
                    'healing': 1,
                    'icon': surface_manager.add_icon('cure minor wounds',
                                                     ':minor_healing.png',
                                                     ''),
                    'title': 'Cure minor wounds',
                    'description': 'Cures small amount of damage'}))

    config.append(('cure medium wounds',
                   {'type': Heal,
                    'duration': 20,
                    'frequency': 5,
                    'tick': 2,
                    'healing': 2,
                    'icon': surface_manager.add_icon('cure medium wounds',
                                                     ':moderate_healing.png',
                                                     ''),
                    'title': 'Cure medium wounds',
                    'description': 'Cures medium amount of damage'}))

    config.append(('minor poison',
                   {'type': Poison,
                    'duration': 240,
                    'frequency': 60,
                    'tick': 60,
                    'damage': 1,
                    'icon': surface_manager.add_icon('minor poison',
                                                     ':minor_poison.png',
                                                     ''),
                    'title': 'Minor poison',
                    'description': 'Causes minor amount of damage'}))

    config.append(('major fire damage',
                   {'type': Damage,
                    'duration': None,
                    'frequency': None,
                    'tick': None,
                    'damage': 12,
                    'damage_type': 'fire',
                    'icon': surface_manager.add_icon('minor poison',
                                                     ':minor_poison.png',
                                                     ''),
                    'title': 'Major fire damage',
                    'description': 'Causes major amount of fire damage'}))

    return config



#!/usr/bin/env python

from distutils.core import setup

setup(name = 'herculeum',
      version = '0.9',
      description = 'Small roguelike game',
      author = 'Tuukka Turto',
      author_email = 'tuukka.turto@oktaeder.net',
      url = 'https://github.com/tuturto/pyherc/',
      classifiers = ['Development Status :: 3 - Alpha',
                     'Environment :: Console :: Curses',
                     'Environment :: Win32 (MS Windows)',
                     'Environment :: X11 Applications :: Qt',
                     'Intended Audience :: End Users/Desktop',
                     'License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)',
                     'Natural Language :: English',
                     'Operating System :: OS Independent',
                     'Programming Language :: Python',
                     'Programming Language :: Python :: 3',
                     'Topic :: Games/Entertainment :: Role-Playing'],
      packages = ['herculeum', 
                 'herculeum.config', 'herculeum.config.levels',
                 'herculeum.ui', 
                 'herculeum.ui.controllers', 'herculeum.ui.gui', 'herculeum.ui.text',
                 'pyherc',
                 'pyherc.ai',
                 'pyherc.config', 'pyherc.config.dsl',
                 'pyherc.data', 'pyherc.data.effects', 'pyherc.data.magic',
                 'pyherc.events',
                 'pyherc.generators', 'pyherc.generators.level', 
                 'pyherc.generators.level.decorator', 'pyherc.generators.level.partitioners',
                 'pyherc.generators.level.room',
                 'pyherc.rules', 'pyherc.rules.attack', 'pyherc.rules.consume',
                 'pyherc.rules.inventory', 'pyherc.rules.move'],
      scripts = ['src/scripts/herculeum'],
      package_data={'pyherc': ['ai/*.hy']},
      requires = ['decorator (>=3.4.0)',
                  'hy (>=0.9.10)'],
      package_dir = {'herculeum': 'src/herculeum',
                     'pyherc': 'src/pyherc'})



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for inventory screen
"""

from pyherc.aspects import logged
from herculeum.ui.controllers import InventoryController

class InventoryScreen():
    """
    Class for displaying inventero screen

    .. versionadded:: 0.9
    """
    @logged
    def __init__(self, character, config, screen, action_factory,
                 parent):
        """
        Default constructor
        """
        super(InventoryScreen, self).__init__()

        self.character = character
        self.config = config
        self.screen = screen.subwin(20, 75, 2, 2)
        self.inventory_controller = InventoryController(character,
                                                        action_factory)
        self.parent = parent

        self.keys = ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j',
                     'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't',
                     'u', 'v', 'w', 'x', 'y', 'z', 'A', 'B', 'C', 'D']

    @logged
    def _draw_screen(self):
        """
        Update the screen
        """
        self.screen.clear()
        self.screen.border()

        for index, item in enumerate(self.character.inventory):
            if index < 15:
                column = 1
                row = 1 + index
            else:
                column = 37
                row = 1 + index - 15

            self.screen.addstr(row, column,
                               '{0} {1}'.format(self.keys[index],
                                                item.get_name(self.character,
                                                              True)))

        self.screen.refresh()

    @logged
    def show(self):
        """
        Show inventory screen
        """
        running = 1
        self._draw_screen()

        while running == 1:
            item = None
            key = chr(self.screen.getch())
            if key in ['u', 'r', 'd', 'i']:
                self.screen.addstr(0, 2, 'select item')
                item_key = chr(self.screen.getch())
                if item_key in self.keys:
                    index = self.keys.index(item_key)
                    if index < len(self.character.inventory):
                        item = self.character.inventory[index]

                if item != None:
                    if key == 'u':
                        self.inventory_controller.use_item(item)
                    elif key == 'r':
                        self.inventory_controller.unequip_item(item)
                    elif key == 'd':
                        self.inventory_controller.drop_item(item)
                    elif key == 'i':
                        self.screen.border()
                        self.screen.addstr(0, 2, 'press key to continue')
                        self.screen.refresh()
                        self._show_detailed_info(item, self.character)
            elif key == ' ':
                running = 0

            self._draw_screen()
            self.parent.refresh()

    @logged
    def _show_detailed_info(self, item, character):
        """
        Show detailed information screen for item

        :param item: item to show
        :type item: Item
        """
        new_screen = self.screen.derwin(18, 73, 1, 1)
        new_screen.clear()

        new_screen.addstr(0, 0, self.inventory_controller.item_description(item))

        new_screen.refresh()

        chr(new_screen.getch())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for Factory for creating effects
"""
from pyherc.aspects import logged
import copy

class EffectsFactory():
    """
    Factory for creating effects
    """
    @logged
    def __init__(self):
        """
        Default constructor
        """
        super(EffectsFactory, self).__init__()
        self.effects = {}

    @logged
    def add_effect(self, key, type):
        """
        Add effect to internal dictionary

        Args:
            key: name of the effect
            type: type used for effect
        """
        self.effects[key] = type

    @logged
    def create_effect(self, key, **kwargs):
        """
        Instantiates new effect with given parameters

        Args:
            key: name of the effect
            kwargs: keyword arguments, passed to the effect
        """
        config = self.effects[key]

        params = copy.deepcopy(config)

        type = params.pop('type')
        if params != None:
            for kw_key in kwargs:
                if kw_key in params:
                    params.pop(kw_key)
                params[kw_key] = kwargs[kw_key]
        else:
            params = kwargs

        return type(**params)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Classes for generating pillar rooms
"""

from pyherc.generators.level.room.squareroom import SquareRoomGenerator
from random import Random
from pyherc.aspects import logged

class PillarRoomGenerator():
    """
    Class for generating a pillar room

    .. versionadded:: 0.8
    """
    @logged
    def __init__(self, floor_tile, empty_tile, pillar_tile, level_types):
        """
        Default constructor

        :param floor_tile: id of the tile to use for floors
        :type floor_tile: integer
        :param empty_tile: id of the empty wall tile
        :type empty_tile: integer
        :param level_types: types of level this generator can be used
        :type level_types: [string]
        """
        self.square_generator = SquareRoomGenerator(floor_tile,
                                                    empty_tile,
                                                    level_types)
        self.floor_tile = floor_tile
        self.empty_tile = empty_tile
        self.level_types = level_types
        self.pillar_tile = pillar_tile
        self.rng = Random()

    @logged
    def generate_room(self, section):
        """
        Generate room

        :param section: section for generator to draw to
        :type section: Section
        """
        self.square_generator.generate_room(section)

        offset = [(1, 1), (-1, 1), (-1, -1), (1, -1)]

        for index, corner in enumerate(self.square_generator.room_corners):
            self.add_pillar(section, corner, offset[index])

    @logged
    def add_pillar(self, section, corner, pillar):
        """
        Add pillar if location is free
        """
        location = (corner[0] + pillar[0],
                    corner[1] + pillar[1])

        if section.get_wall(location) == self.empty_tile:
            section.set_wall(location = location,
                             tile = self.pillar_tile,
                             location_type = None)
            section.set_location_type(location, None)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Classes for creature generation
"""

from pyherc.aspects import logged

class CreatureAdderConfiguration():
    """
    Class used to configure CreatureAdder
    """
    @logged
    def __init__(self, level_types):
        """
        Default constructor

        :param level_types: types of levels adder can be used at
        :type level_types: [string]
        """
        super(CreatureAdderConfiguration, self).__init__()
        self.level_types = level_types
        self.creature_list = []

    @logged
    def add_creature(self, min_amount, max_amount, name, location = None):
        """
        Adds creature specification

        :param min_amount: minimum amount of creatures to generate
        :type min_amount: integer
        :param max_amount: maximum amount of creatures to generate
        :type max_amount: integer
        :param name: name of the creature to generate
        :type name: string
        :param location: location type where creature is placed
        :type location: string
        """
        config_item = {}
        config_item['min_amount'] = min_amount
        config_item['max_amount'] = max_amount
        config_item['name'] = name
        config_item['location'] = location

        self.creature_list.append(config_item)

class CreatureAdder():
    """
    Class used to add creatures during level generation
    """
    @logged
    def __init__(self, creature_generator, configuration, rng):
        """
        Default constructor

        :param creature_generator: creature generator used to create creatures
        :type creature_generator: CreatureGenerator
        :param configuration: configuration
        :type configuration: CreatureAdderConfiguration
        :param rng: random number generator
        :type rng: Random
        """
        super(CreatureAdder, self).__init__()
        self.creature_generator = creature_generator
        self.configuration = configuration
        self.rng = rng

    @logged
    def __get_level_types(self):
        """
        Get level types this adder can be used at

        :returns: level types this adder can be used at
        :rtype: [string]
        """
        return self.configuration.level_types

    @logged
    def add_creatures(self, level):
        """
        Add creatures to level according to configuration

        :param level: level to add creatures
        :type level: Level
        """
        creature_list = self.configuration.creature_list
        creatures = []

        for creature in creature_list:
            amount = self.rng.randint(creature['min_amount'],
                                      creature['max_amount'])
            creatures.extend(self.generate_creatures(creature['name'],
                                                     amount))

        self.place_creatures(creatures, creature_list, level)

    @logged
    def generate_creatures(self, name, amount):
        """
        Generate creatures

        :param name: name of the creatures to generate
        :type name: string
        :param amount: amount of creatures to generate
        :type amount: integer
        :returns: generated creatures
        :rtype: [Character]
        """
        creatures = []
        for i in range(amount):
            new_creature = self.creature_generator.generate_creature(name)
            creatures.append(new_creature)

        return creatures

    @logged
    def place_creatures(self, creatures, creature_list, level):
        """
        Place creatures into a level

        :param creatures: creatures to place
        :type creatures: [Character]
        :param creature_list: specification where to place creatures
        :type creature_list: dict
        :param level: level to place creatures
        :type level: Level
        """
        for creature in creatures:
            location_type = [x['location'] for x in creature_list
                             if x['name'] == creature.name]

            if location_type == None:
                location_type = 'any'

            locations = level.get_locations_by_type('room')

            location = self.rng.choice(locations)

            level.add_creature(creature, location)

    level_types = property(__get_level_types)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for main map related functionality
"""
from PyQt4.QtGui import QWidget, QHBoxLayout, QVBoxLayout
from PyQt4.QtGui import QGraphicsPixmapItem, QGraphicsView, QGraphicsScene
from PyQt4.QtGui import QGraphicsSimpleTextItem, QColor
from PyQt4.QtGui import QFont
from PyQt4.QtCore import QSize, Qt, QPropertyAnimation, QObject, pyqtProperty
from PyQt4.QtCore import QAbstractAnimation, QSequentialAnimationGroup
from PyQt4.QtCore import QEasingCurve, pyqtSignal, QEvent
from herculeum.ui.gui.eventdisplay import EventMessageWidget
from herculeum.ui.gui.widgets import HitPointsWidget, EffectsWidget
from herculeum.ui.controllers import MoveController
from random import Random

class PlayMapWindow(QWidget):
    """
    Window for displaying playing world

    .. versionadded:: 0.5
    """
    def __init__(self, parent, model, surface_manager, action_factory, rng,
                 rules_engine, configuration):
        """
        Default constructor
        """
        super(PlayMapWindow, self).__init__(parent)

        self.model = model
        self.surface_manager = surface_manager
        self.action_factory = action_factory
        self.rng = rng
        self.current_level = None
        self.configuration = configuration
        
        self.hit_points_widget = None
        self.message_widget = None
        self.map_widget = None
        self.effects_widget = None

        self.__set_layout(model, surface_manager, action_factory, rng,
                          rules_engine, configuration)

    MenuRequested = pyqtSignal(name='MenuRequested')
    EndScreenRequested = pyqtSignal(name='EndScreenRequested')

    def __set_layout(self, model, surface_manager, action_factory, rng,
                     rules_engine, configuration):
        """
        Set layout of this window
        """
        layout = QVBoxLayout()
        status_layout = QHBoxLayout()

        self.hit_points_widget = HitPointsWidget(parent = self)
        self.effects_widget = EffectsWidget(parent = self,
                                            surface_manager = surface_manager)

        self.map_widget = PlayMapWidget(parent = self,
                                        model = model,
                                        surface_manager = surface_manager,
                                        action_factory = action_factory,
                                        rng = rng,
                                        rules_engine = rules_engine,
                                        configuration = configuration)
        self.map_widget.MenuRequested.connect(self.on_menu_requested)
        self.map_widget.EndScreenRequested.connect(self.on_end_screen_requested)

        self.message_widget = EventMessageWidget(parent = self)
        self.message_widget.setMaximumHeight(100)

        status_layout.addWidget(self.hit_points_widget)
        status_layout.addWidget(self.effects_widget)
        status_layout.addStretch()

        layout.addLayout(status_layout)
        layout.addWidget(self.map_widget)
        layout.addWidget(self.message_widget)
        self.setLayout(layout)
        self.resize(QSize(640, 480))

    def construct_scene(self):
        """
        Create scene to display
        """
        self.map_widget.construct_scene()
        self.model.player.register_event_listener(self.message_widget)
        self.message_widget.set_point_of_view(self.model.player)
        self.model.player.register_for_updates(self.effects_widget)

    def on_menu_requested(self):
        """
        Handle requesting menu window
        """
        self.MenuRequested.emit()

    def on_end_screen_requested(self):
        """
        Handle requesting end screen

        .. versionadded:: 0.8
        """
        self.EndScreenRequested.emit()

class PlayMapWidget(QWidget):
    """
    Widget for displaying playing world

    .. versionadded:: 0.5
    """
    def __init__(self, parent, model, surface_manager, action_factory, rng,
                 rules_engine, configuration):
        """
        Default constructor
        """
        super(PlayMapWidget, self).__init__(parent)

        self.model = model
        self.scene = None
        self.surface_manager = surface_manager
        self.action_factory = action_factory
        self.rng = rng
        self.rules_engine = rules_engine
        self.configuration = configuration
        
        self.current_level = None
        self.view = None

        self.animations = []
        self.move_controller = MoveController(action_factory = action_factory,
                                              rng = rng)

        self.__set_layout()
        self.keymap, self.move_key_map = self._construct_keymaps(
                                                        configuration.controls)

    MenuRequested = pyqtSignal(name='MenuRequested')
    EndScreenRequested = pyqtSignal(name='EndScreenRequested')

    def _construct_keymaps(self, config):
        """
        Construct keymaps for handling input
        """
        keymap = {}
        move_keymap = {}
        for key in config.move_left:
            keymap[key] = self._move
            move_keymap[key] = 7
        for key in config.move_up_left:
            keymap[key] = self._move
            move_keymap[key] = 8
        for key in config.move_up:
            keymap[key] = self._move
            move_keymap[key] = 1
        for key in config.move_up_right:
            keymap[key] = self._move
            move_keymap[key] = 2
        for key in config.move_right:
            keymap[key] = self._move
            move_keymap[key] = 3
        for key in config.move_down_right:
            keymap[key] = self._move
            move_keymap[key] = 4
        for key in config.move_down:
            keymap[key] = self._move
            move_keymap[key] = 5
        for key in config.move_down_left:
            keymap[key] = self._move
            move_keymap[key] = 6
        for key in config.start:
            keymap[key] = self._menu
        for key in config.action_a:
            keymap[key] = self._action_a

        return keymap, move_keymap

    def __set_layout(self):
        """
        Set layout of this widget
        """
        self.scene = QGraphicsScene()

        layout = QHBoxLayout()

        self.view = QGraphicsView(self.scene)
        self.view.setFocusPolicy(Qt.StrongFocus)
        self.view.installEventFilter(self)
        layout.addWidget(self.view)

        self.setLayout(layout)

    def construct_scene(self):
        """
        Construct scene to display
        """
        self.__construct_scene(self.model, self.scene)

        self.model.player.register_for_updates(self)
        self.model.register_event_listener(self)
        self.__center_view_on_character(self.model.player)

    def __center_view_on_character(self, entity):
        """
        Center view on given entity
        """
        location = entity.location
        width = 320
        height = 320

        self.view.setSceneRect((location[0] * 32) - width // 2,
                              (location[1] * 32) - height // 2,
                              width,
                              height)

    def __construct_scene(self, model, scene):
        """
        Constructs scene to display
        """
        for item in (item for item in scene.items()
                     if hasattr(item, 'clear_update_registration')):
            item.clear_update_registration()

        for anim in [x for x in self.animations]:
            anim.clear()

        self.animations = []

        scene.clear()

        self.current_level = model.player.level

        for loc_x, column in enumerate(self.current_level.floor):
            for loc_y, tile in enumerate(column):
                new_glyph = MapGlyph(self.surface_manager.get_icon(tile),
                                     None)
                new_glyph.setZValue(0)
                new_glyph.setPos(loc_x * 32, loc_y * 32)
                scene.addItem(new_glyph)

        for loc_x, column in enumerate(self.current_level.walls):
            for loc_y, tile in enumerate(column):
                new_glyph = MapGlyph(self.surface_manager.get_icon(tile),
                                     None)
                new_glyph.setZValue(1)
                new_glyph.setPos(loc_x * 32, loc_y * 32)
                scene.addItem(new_glyph)

        for portal in self.current_level.portals:
            self.add_glyph(portal, scene, 10)

        for item in self.current_level.items:
            self.add_glyph(item, scene, 20)

        for creature in self.current_level.creatures:
            self.add_glyph(creature, scene, 30)

    def add_glyph(self, entity, scene, z_order):
        """
        Add graphical representation of an entity

        :param entity: entity to display
        :param scene: scene where glyph will be added
        :type scene: QGraphicsScene
        :param z_order: z-order of entity being displayed
        :type z_order: int
        """
        new_glyph = MapGlyph(self.surface_manager.get_icon(entity.icon),
                             entity)
        new_glyph.setZValue(z_order)
        new_glyph.setPos(entity.location[0] * 32,
                         entity.location[1] * 32)
        scene.addItem(new_glyph)

    def receive_event(self, event):
        """
        Receive event from model
        """
        if event.event_type == 'death':
            glyphs = [x for x in self.view.items()
                      if (hasattr(x, 'entity'))
                      and (x.entity == event.deceased)]

            for glyph in glyphs:
                self.view.scene().removeItem(glyph)

            if event.deceased == self.model.player:
                self.EndScreenRequested.emit()
        elif event.event_type == 'pick up':
            glyphs = [x for x in self.view.items()
                      if (hasattr(x, 'entity'))
                      and (x.entity == event.item)]

            for glyph in glyphs:
                self.view.scene().removeItem(glyph)

        elif event.event_type == 'drop':
            self.add_glyph(event.item, self.scene, 20)
        elif event.event_type == 'attack hit':
            damage = event.damage.damage_inflicted
            self.show_damage_counter(event.target.location,
                                     -damage,
                                     'white')
        elif event.event_type == 'damage triggered':
            damage = event.damage
            self.show_damage_counter(event.target.location,
                                     -damage,
                                     'red',
                                     (0, 16))
        elif event.event_type == 'poisoned':
            self.show_status_counter(event.target.location,
                                     'poisoned',
                                     'green',
                                     (0, 16))
        elif event.event_type == 'poison triggered':
            self.show_damage_counter(event.target.location,
                                     -event.damage,
                                     'green',
                                     (0, 16))
        elif event.event_type == 'heal started':
            self.show_status_counter(event.target.location,
                                     'healing',
                                     'blue',
                                     (0, 16))
        elif event.event_type == 'heal triggered':
            self.show_damage_counter(event.target.location,
                                     event.healing,
                                     'blue',
                                     (0, 16))
        elif event.event_type == 'notice':
            self.show_status_counter(event.character.location,
                                     '!',
                                     'red',
                                     (0, 16))
        elif event.event_type == 'lose focus':
            self.show_status_counter(event.character.location,
                                     '?',
                                     'yellow',
                                     (0, 16))

    def show_status_counter(self, location, status, colour, offset = (0, 0)):
        """
        Show status counter
        """
        damage_counter = DamageCounter(damage = str(status),
                                       colour = colour,
                                       parent = self)
        self.view.scene().addItem(damage_counter)

        bounds = damage_counter.boundingRect()
        width = bounds.width()

        damage_counter.setPos(location[0] * 32 + 16 - (width / 2) + offset[0],
                              location[1] * 32 + offset[1])

        animation = QSequentialAnimationGroup()

        moving = QPropertyAnimation(damage_counter.adapter,
                                    'y_location')
        moving.setDuration(750)
        moving.setStartValue(location[1] * 32)
        moving.setEndValue(location[1] * 32 - 32)

        animation.addAnimation(moving)

        fading = QPropertyAnimation(damage_counter.adapter,
                                    'opacity')
        fading.setDuration(750)
        fading.setStartValue(1.0)
        fading.setEndValue(0.0)
        animation.addAnimation(fading)

        animation.finished.connect(self.remove_finished_animation)
        self.animations.append(animation)

        animation.start()

    def show_damage_counter(self, location, damage, colour, offset = (0, 0)):
        """
        Show damage counter
        """
        damage_counter = DamageCounter(damage = str(damage),
                                       colour = colour,
                                       parent = self)
        self.view.scene().addItem(damage_counter)

        bounds = damage_counter.boundingRect()
        width = bounds.width()

        rand = Random()

        damage_counter.setPos((location[0] * 32
                                    + 16 - (width / 2)
                                    + rand.randint(-16, 16))
                                    + offset[0],
                              location[1] * 32 + offset[1])

        animation = QSequentialAnimationGroup()

        moving = QPropertyAnimation(damage_counter.adapter,
                                    'y_location')
        moving.setDuration(750)
        moving.setStartValue(location[1] * 32 + offset[1])
        moving.setEndValue(location[1] * 32 - 32 + offset[1])
        curve = QEasingCurve(QEasingCurve.OutElastic)
        moving.setEasingCurve(curve)
        animation.addAnimation(moving)

        fading = QPropertyAnimation(damage_counter.adapter,
                                    'opacity')
        fading.setDuration(750)
        fading.setStartValue(1.0)
        fading.setEndValue(0.0)
        animation.addAnimation(fading)

        animation.finished.connect(self.remove_finished_animation)
        self.animations.append(animation)

        animation.start()

    def remove_finished_animation(self):
        """
        Remove finished animation
        """
        finished_animations  = (x for x in self.animations
                                if x.state() == QAbstractAnimation.Stopped)
        counters = (x.animationAt(0).targetObject().object_to_animate
                    for x in finished_animations)

        for item in finished_animations:
            item.clear()
            self.animations.remove(item)

        for item in counters:
            self.view.scene().removeItem(item)

    def receive_update(self, event):
        """
        Receive update from entity
        """
        if event.event_type == 'move':
            if self.model.player.level != self.current_level:
                self.__construct_scene(self.model, self.scene)
            self.__center_view_on_character(self.model.player)

    def eventFilter(self, qobject, event): #pylint: disable-msg=C0103
        """
        Filter events

        .. Note:: This is done in order to process cursor keys
        """
        result = False

        if event.type() == QEvent.KeyPress:
            self.keyPressEvent(event)
            result = True
        else:
            result = super(PlayMapWidget, self).eventFilter(qobject, event)

        return result

    def keyPressEvent(self, event): #pylint: disable-msg=C0103
        """
        Handle key events
        """
        if self.model.player is None:
            return

        key_code = event.key()

        player = self.model.player
        next_creature = self.model.get_next_creature(self.rules_engine)

        if next_creature == player:

            if key_code in self.keymap:
                self.keymap[key_code](key_code, event.modifiers())

        next_creature = self.model.get_next_creature(self.rules_engine)
        while next_creature != player and next_creature != None:
            next_creature.act(model = self.model,
                              action_factory = self.action_factory,
                              rng = self.rng)
            next_creature = self.model.get_next_creature(self.rules_engine)

        if self.model.end_condition != 0:
            self.EndScreenRequested.emit()

    def _move(self, key, modifiers):
        """
        Process movement key

        :param key: key triggering the processing
        :type key: int
        """
        player = self.model.player
        direction = self.move_key_map[key]

        if modifiers & Qt.ControlModifier:
            if direction != 9:
                player.perform_attack(direction,
                                      self.action_factory,
                                      self.rng)
        else:
            self.move_controller.move_or_attack(player,
                                                direction,
                                                'walk')

    def _menu(self, key, modifiers):
        """
        Process menu key

        :param key: key triggering the processing
        :type key: int
        """
        self.MenuRequested.emit()

    def _action_a(self, key, modifiers):
        """
        Process action a key

        :param key: key triggering the processing
        :type key: int
        """
        player = self.model.player
        level = player.level
        items = level.get_items_at(player.location)

        if items != None and len(items) > 0:
            player.pick_up(items[0],
                           self.action_factory)

        elif player.is_move_legal(9,
                                  'walk',
                                  self.action_factory):
            player.move(9,
                        self.action_factory)

class DamageCounter(QGraphicsSimpleTextItem):
    """
    Counter for showing damage

    .. versionadded:: 0.6
    """
    def __init__(self, damage, colour, parent):
        """
        Default constructor
        """
        super(DamageCounter, self).__init__()

        font = QFont('Helvetica',
                     12,
                     QFont.Bold,
                     False)

        self.setText(str(damage))
        self.setBrush(QColor(colour))
        self.setFont(font)

        self.adapter = DamageCounterAdapter(self, self)

class DamageCounterAdapter(QObject):
    """
    Adapter for damage counter

    .. versionadded:: 0.6
    """
    def __init__(self, parent, object_to_animate):
        """
        Default constructor
        """
        super(DamageCounterAdapter, self).__init__()
        self.object_to_animate = object_to_animate

    def __get_y_location(self):
        return self.object_to_animate.y()

    def __set_y_location(self, y):
        self.object_to_animate.setY(y)

    def __get_opacity(self):
        return self.object_to_animate.opacity()

    def __set_opacity(self, opacity):
        self.object_to_animate.setOpacity(opacity)

    y_location = pyqtProperty(int, __get_y_location, __set_y_location)
    opacity = pyqtProperty(float, __get_opacity, __set_opacity)


class MapGlyph(QGraphicsPixmapItem):
    """
    Widget to represent a glyph on map

    .. versionadded:: 0.5
    """
    def __init__(self, pixmap, entity):
        """
        Default constructor
        """
        super(MapGlyph, self).__init__(pixmap, None)

        self.entity = entity

        if entity != None:
            entity.register_for_updates(self)

    def receive_update(self, event):
        """
        Receive event from model
        """
        if event.event_type == 'move':
            location = event.mover.location
            if location != None:
                self.setPos(location[0] * 32,
                            location[1] * 32)

    def clear_update_registration(self):
        """
        Clear update registrations
        """
        if self.entity != None:
            self.entity.remove_from_updates(self)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for Inventory related classes
"""

from pyherc.aspects import logged

class Inventory():
    """
    Represents an inventory of items
    """
    @logged
    def __init__(self):
        """
        Default constructor
        """
        super(Inventory, self).__init__()

        self.__items = []

        self.ring = None
        self.weapon = None
        self.gloves = None
        self.boots = None
        self.belt = None
        self.helm = None
        self.necklace = None
        self.projectiles = None
        self.shield = None
        self.armour = None

    def __len__(self):
        """
        Length of the container
        """
        return len(self.__items)

    def __getitem__(self, key):
        """
        Access an item in Inventory
        """
        return self.__items.__getitem__(key)

    def __setitem__(self, key, value):
        """
        Set item in inventory
        """
        self.__items.__setitem__(key, value)

    def __delitem__(self, key):
        """
        Delete item from inventory
        """
        item = self.__items[key]
        self.remove(item)

    def __iter__(self):
        """
        Get iterator for content of inventory
        """
        return self.__items.__iter__()

    def append(self, item):
        """
        Append an item into inventory
        """
        self.__items.append(item)

    def remove(self, item):
        """
        Remove item from inventory

        :param item: item to remove
        :type item: Item
        """
        self.__items.remove(item)

        if item == self.weapon:
            self.weapon = None
        if item == self.armour:
            self.armour = None
        if item == self.projectiles:
            self.projectiles = None

    def _repr_pretty_(self, p, cycle):
        """
        Pretty print for IPython

        :param p: printer to write
        :param cycle: has pretty print detected a cycle?
        """
        if cycle:
            p.text('Inventory(...)')
        else:
            p.text('Inventory:')
            for item in self.__items:
                p.pretty(item)
                p.breakable()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
module for RoomGenerators
"""
from pyherc.generators.level.room.catacombs import CatacombsGenerator
from pyherc.generators.level.prototiles import FLOOR_NATURAL
from random import Random

class Catacombs():
    """
    Generator for catacombs
    """
    def __init__(self):
        """
        Default constructor
        """
        super(Catacombs, self).__init__()
        self.floor_tile = 0
        self.empty_tile = 0
        self.rng = Random()
        self.locations = []

    def with_(self, parameter):
        """
        set parameter for room generation

        :param parameter: parameter to set
        :type parameter: (string, value)
        """
        if hasattr(parameter, 'count'):
            if parameter[0] == 'floor':
                self.floor_tile = parameter[1]

        elif hasattr(parameter, 'random'):
            self.rng = parameter

        return self

    def located_at(self, location):
        """
        Set location of room

        :param location: name of location
        :type location: string
        """
        self.locations.append(location)
        return self

    def build(self):
        """
        Build generator

        :returns: configured generator
        :rtype: CatacombsGenerator
        """
        return CatacombsGenerator(floor_tile = self.floor_tile,
                                  empty_tile = self.empty_tile,
                                  level_types = self.locations,
                                  rng = self.rng)

def natural_floor():
    """
    Get definition of natural floor

    :returns: definition of natural floor
    :rtype: (string, int)
    """
    return ('floor', FLOOR_NATURAL)
